# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from pyforms_generic_editor.models.projects.projects_treenode import ProjectsTreeNode

logger = logging.getLogger(__name__)


class ProjectsDockWindow(ProjectsTreeNode):
	def __init__(self, mainwindow=None):
		ProjectsTreeNode.__init__(self, mainwindow)

		self.mainwindow = mainwindow
		self.mainwindow.dock.value = self

		self.register_on_toolbar(self.mainwindow.toolbar)
		self.register_on_main_menu(self.mainwindow.mainmenu)

	def register_on_toolbar(self, toolbar):
		pass

	def register_on_main_menu(self, mainmenu):
		filemenu = mainmenu[0]
		filemenu['File'].insert(0, {'New project': self.create_project, 'icon': conf.NEW_SMALL_ICON})
		filemenu['File'].insert(1, '-')
		filemenu['File'].insert(2, {'Open a project': self._option_open_project, 'icon': conf.OPEN_SMALL_ICON})
		filemenu['File'].insert(3, '-')
		filemenu['File'].insert(4,
		                        {'Save current project': self.save_current_project, 'icon': conf.SAVE_SMALL_ICON})
		filemenu['File'].insert(5, {'Save current project as': self.save_current_project_as,
		                            'icon': conf.SAVE_SMALL_ICON})
		filemenu['File'].insert(6, {'Save all projects': self.save_all_projects, 'icon': conf.SAVE_SMALL_ICON})
		filemenu['File'].insert(7, '-')

	def _option_open_project(self):
		try:
			self.open_project()
		except Exception as err:
			for project in self.projects:
				project.close(silent=True)

			logger.warning(str(err), exc_info=True)
			self.warning(str(err), "Invalid project path")



import os
import logging
import pybpodgui_api
from pybpodgui_api.models.user.user_base import UserBase

from sca.formats import json

logger = logging.getLogger(__name__)


class UserIO(UserBase):

    def __init__(self, _project):
        super(UserIO, self).__init__(_project)

        self.data = None  # DOn't know if we actualy need this

    def save(self):
        if not self.name:
            logger.warning('Skipping user without a name')
            return None
        else:
            if not os.path.exists(self.path):
                os.makedirs(self.path)

            if self.data:
                data = self.data
            else:
                data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a user used on PyBpod GUI.'
                )

            config_path = os.path.join(self.path, self.name + '.json')
            with open(config_path, 'w') as fstream:
                json.dump(data, fstream)

    def toJSON(self):
        data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a user used on PyBpod GUI.'
                )

        return json.dumps(data)

    def load(self, path):
        self.name = os.path.basename(path)

        try:
            with open(os.path.join(self.path, self.name + '.json'), 'r') as stream:
                self.data = data = json.load(stream)
            self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4
        except:
            raise Exception('There was an error loading the configuration file for the subject [{0}]')

    def collect_data(self, data):
        data.update({'name': self.name})
        return data



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class GlobalTimers(object):
    def __init__(self, max_states, n_global_timers):

        self.start_matrix = [[] for i in range(max_states)]
        self.end_matrix = [[] for i in range(max_states)]

        # for each state\position has an integer which the bits represents global timers to trigger
        self.triggers_matrix = [0 for i in range(max_states)]

        # for each state\position has an integer which the bits represents global timers to cancel
        self.cancels_matrix = [0 for i in range(max_states)]

        # for each state\position has an integer which the bits represents global timers to on set
        # TODO count the number of timers used on the state machine
        self.onset_matrix = [0] * n_global_timers

        self.timers = [0] * n_global_timers
        self.on_set_delays = [0] * n_global_timers
        self.channels = [255] * n_global_timers
        self.on_messages = [255] * n_global_timers
        self.off_messages = [255] * n_global_timers

        self.loop_mode = [0] * n_global_timers
        self.loop_intervals = [0] * n_global_timers
        self.send_events = [1] * n_global_timers

        self.max_size = n_global_timers

    def get_max_index_used(self):
        for i in range(self.max_size - 1, -1, -1):
            if self.timers[i] != 0:
                return i
        return None



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodgui_api.models.setup import Setup

from pybpodgui_plugin.models.board.board_window import BoardWindow
from AnyQt.QtCore import QTimer

logger = logging.getLogger(__name__)


class BoardCom(BoardWindow):
    """
    Board communication logic. Define here actions that can be triggered on board.

    .. seealso::
        This class heavy relies on the corresponding API module.

        :py:class:`pybpodgui_api.models.board.board_com.BoardCom`

    **Methods**

    """

    def __init__(self, project=None):
        BoardWindow.__init__(self, project)

        self._timer = QTimer()
        self._timer.timeout.connect(self.run_task_handler)

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################

    def log2board(self, data):
        if hasattr(self, '_log') and self._log.visible:
            self._log += data

    def run_task(self, session, board_task, workspace_path, detached=False):
        """
        Bases: :meth:`pybpodgui_api.models.board.board_com.BoardCom.run_task`

        Start running task on board by invoking API

        :param session:
        :param board_task: board and task object
        :return: True if no problems occur, False otherwise.
        """
        flag = None
        self._enable_btn_flag = True
        self._tmp_setup = session.setup
        try:
            flag = super(BoardCom, self).run_task(session, board_task, workspace_path, detached)
        except Exception:
            board_task.setup.status = Setup.STATUS_READY
            self.status = self.STATUS_READY
            raise

        return flag

    def start_run_task_handler(self):
        super(BoardCom, self).start_run_task_handler()
        self._timer.start(500)

    def end_run_task_handler(self):
        super(BoardCom, self).end_run_task_handler()
        self.run_task_handler(False)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from pybpodapi.state_machine.state_machine_builder import StateMachineBuilder

logger = logging.getLogger(__name__)


class StateMachineRunner(StateMachineBuilder):
    """
    Extends state machine with running logic

    :ivar bool is_running: Whether this state machine is being run on bpod hardware
    :ivar int current_state: Holds state machine current state while running
    """

    def __init__(self, bpod):
        StateMachineBuilder.__init__(self, bpod)

        self.is_running = False  # type: bool

        self.current_state = 0  # type: int

    #########################################
    ############## PROPERTIES ###############
    #########################################

    @property
    def is_running(self):
        return self._is_running  # type: bool

    @is_running.setter
    def is_running(self, value):
        self._is_running = value  # type: bool

    @property
    def current_state(self):
        return self._current_state  # type: int

    @current_state.setter
    def current_state(self, value):
        self._current_state = value  # type: int


class StateMachineRunnerError(Exception):
    pass



SETTINGS_PRIORITY = 0

PYBPOD_SERIAL_PORT       = '{serialport}'
PYBPOD_NET_PORT          = {netport}

{bnp_ports}
{wired_ports}
{behavior_ports}

PYBPOD_API_LOG_LEVEL = None
PYBPOD_API_LOG_FILE  = None

PYBPOD_API_STREAM2STDOUT = {stream2stdout}
PYBPOD_API_ACCEPT_STDIN  = True

PYBPOD_PROTOCOL 	= '{protocolname}'
PYBPOD_CREATOR 		= '{user}'
PYBPOD_PROJECT 		= '{project}'
PYBPOD_EXPERIMENT 	= '{experiment}'
PYBPOD_BOARD 		= '{board}'
PYBPOD_SETUP 		= '{setup}'
PYBPOD_SESSION 		= '{session}'
PYBPOD_SESSION_PATH = '{session_path}'
PYBPOD_SUBJECTS 	= [{subjects}]
PYBPOD_SUBJECT_EXTRA = '{subject_extra}'
PYBPOD_USER_EXTRA = '{user_extra}'


TARGET_BPOD_FIRMWARE_VERSION = '{bpod_firmware_version}'

#import logging
#PYBPOD_API_LOG_LEVEL = logging.DEBUG
#PYBPOD_API_LOG_FILE  = 'pybpod-api.log'

PYBPOD_VARSNAMES = [{variables_names}]


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

import pyforms as app
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText
from pyforms.controls import ControlCheckBox
from pyforms.controls import ControlList

from pybpodgui_api.models.task import Task
from pybpodgui_plugin.models.task.windows.command_editor import CommandEditor


from pybpodgui_api.models.task.taskcommand import TaskCommand

logger = logging.getLogger(__name__)


class TaskWindow(Task, BaseWidget):
    """
    Define here which fields from the task model should appear on the details section.

    The model fields shall be defined as UI components like text fields, buttons, combo boxes, etc.

    You may also assign actions to these components.

    **Properties**

        name
            :class:`string`

            Name associated with this task. Returns the current value stored in the :py:attr:`_name` text field.

    **Private attributes**

        _name
            :class:`pyforms.controls.ControlText`

            Text field to edit task name. Editing this field fires the event :meth:`TaskWindow._TaskWindow__name_edited_evt`.

        _edit_btn
            :class:`pyforms.controls.ControlButton`

            Button to edit task code. Pressing the button fires the event :meth:`BoardWindow._BoardWindow__install_framework_btn_evt`.

        _formset
            Describe window fields organization to PyForms.

    **Methods**

    """

    def __init__(self, project=None):
        BaseWidget.__init__(self, 'Task')
        self.layout().setContentsMargins(5, 10, 5, 5)

        self.precmdwin = None
        self.postcmdwin = None

        self._namefield = ControlText('Task name', changed_event=self.__name_edited_evt)
        self._use_server = ControlCheckBox('Trigger soft codes using a UDP port')
        self._precmds = ControlList(
            'Pre commands',
            add_function=self.__add_pre_command,
            remove_function=self.__remove_pre_command,
            readonly=True
        )
        self._postcmds = ControlList(
            'Post commands',
            add_function=self.__add_post_command,
            remove_function=self.__remove_post_command,
            readonly=True
        )

        self._formset = [
            '_namefield',
            '_use_server',
            '_precmds',
            '_postcmds',
            ' '
        ]

        Task.__init__(self, project)

        self.update_commands()

    def update_commands(self):
        self._precmds.clear()
        self._postcmds.clear()

        self._precommands_list = []
        self._postcommands_list = []

        for cmd in self.commands:
            if cmd.when == TaskCommand.WHEN_PRE:
                self._precmds += [str(cmd)]
                self._precommands_list.append(cmd)
            elif cmd.when == TaskCommand.WHEN_POST:
                self._postcmds += [str(cmd)]
                self._postcommands_list.append(cmd)

    def __add_pre_command(self):
        if self.precmdwin is not None:
            self.precmdwin.show()
            self.precmdwin.activateWindow()
            self.precmdwin.raise_()
        else:
            self.precmdwin = CommandEditor(self, when=TaskCommand.WHEN_PRE)
            self.precmdwin.show()

    def __remove_pre_command(self):
        row = self._precmds.selected_row_index
        if row is not None:
            obj = self._precommands_list.pop(row)
            self._commands.remove(obj)
            self._precmds -= -1

    def __add_post_command(self):
        if self.postcmdwin is not None:
            self.postcmdwin.show()
            self.postcmdwin.activateWindow()
            self.postcmdwin.raise_()
        else:
            self.postcmdwin = CommandEditor(self, when=TaskCommand.WHEN_POST)
            self.postcmdwin.show()

    def __remove_post_command(self):
        row = self._postcmds.selected_row_index
        if row is not None:
            obj = self._postcommands_list.pop(row)
            self._commands.remove(obj)
            self._postcmds -= -1

    def __use_server_changed_evt(self):
        self._netport.enabled = True if self._use_server.value else False

    def __name_edited_evt(self):
        """
        React to changes on text field :py:attr:`_name`.

        This methods is called every time the user changes the field.
        """
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._namefield.value

    @property
    def name(self): return Task.name.fget(self)

    @name.setter
    def name(self, value):
        try:
            Task.name.fset(self, value)
        except FileNotFoundError as e:
            self.critical(str(e), 'File not found')
        # Flag to avoid recurse calls when editing the name text field
        self._update_name = True
        self._namefield.value = value
        self._update_name = False

    @property
    def trigger_softcodes(self):
        return self._use_server.value

    @trigger_softcodes.setter
    def trigger_softcodes(self, value):
        self._use_server.value = (value is True)

    def load(self, path):
        super(TaskWindow, self).load(path)
        self.update_commands()


# Execute the application
if __name__ == "__main__":
    app.start_app(TaskWindow)



from one.webclient import AlyxClient
from confapp import conf


class AlyxModule(object):

    def __init__(self):
        self._addr = conf.ALYX_PLUGIN_ADDRESS
        self._api = None

    def _connect_to_alyx(self, username, password):
        try:
            if self._api is None:
                self._api = AlyxClient(username=username, password=password, base_url=self._addr)
        except Exception:
            return False

        return True

    def get_alyx_subjects(self, username):
        return self._api.rest('subjects', 'list', '?responsible_user={name}'.format(name=username))

    def get_alyx_subject_info(self, nickname):
        return self._api.rest('subjects', 'list', '/{nick}'.format(nick=nickname))

    def set_alyx_address(self, value):
        self._addr = value



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf

from pybpodgui_plugin.models.setup.board_task.board_task_uibusy import BoardTaskUIBusy

BoardTask = type(
    'BoardTask',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.setup.board_task.BoardTask') + [BoardTaskUIBusy]),
    {}
)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

import pyforms
from pyforms.basewidget import BaseWidget


logger = logging.getLogger(__name__)


class GenericProject(BaseWidget):
	""" ProjectWindow represents the project entity as a GUI window"""

	def __init__(self):
		BaseWidget.__init__(self, 'Generic Project')
		self.layout().setContentsMargins(5, 10, 5, 5)

	def save(self, project_path=None):
		pass

	def save_as(self):
		pass

	def load(self, project_path):
		pass

	def close(self, silent=False):
		pass


# Execute the application
if __name__ == "__main__":
	pyforms.start_app(GenericProject)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)


class GlobalCounters(object):
    def __init__(self, max_states, n_global_counters):
        self.matrix = [[] for i in range(max_states)]

        self.reset_matrix = [0 for i in range(max_states)]

        self.attached_events = [254] * n_global_counters
        self.thresholds = [0] * n_global_counters

        self.max_size = n_global_counters

    def get_max_index_used(self):

        for i in range(self.max_size - 1, -1, -1):
            if self.thresholds[i] != 0:
                return i

        return None



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon
from AnyQt import QtCore


from pybpodgui_plugin.models.setup.setup_window import SetupWindow


logger = logging.getLogger(__name__)


class SetupTreeNode(SetupWindow):
    """
    Extends setup window to show up in the project tree section.
    Define here actions related to the setup tree node.

    **Properties**

        name
            Handles setup tree node name.

        tree
            Returns project tree.

    **Methods**

    """

    def __init__(self, experiment):
        SetupWindow.__init__(self, experiment)

        self.__running_icon = QIcon(conf.PLAY_SMALL_ICON)

        self.create_treenode(self.tree)

    def create_treenode(self, tree):
        """
        Creates node for this setup under the corresponding experiment parent node.

        This methods is called when the setup is first created.

        The following actions get assigned to node:
            * *Remove*: :meth:`SetupTreeNode.remove`.

        Sets key events:
            * :meth:`SetupTreeNode.node_key_pressed_event`


        :param tree: the project tree
        :type tree: pyforms.controls.ControlTree
        :return: new created node
        :return type: QTreeWidgetItem
        """

        self.node = tree.create_child(self.name, self.experiment.node, icon=QIcon(conf.BOX_SMALL_ICON))
        self.node.key_pressed_event = self.node_key_pressed_event
        self.node.window = self
        # self.node.setExpanded(True)

        tree.add_popup_menu_option('Remove', self.remove, item=self.node, icon=QIcon(conf.REMOVE_SMALL_ICON))
        return self.node

    def remove(self):
        """
        Remove setup from project and remove node from tree.

        .. seealso::
            * Setup removal (dock window): :py:meth:`pybpodgui_plugin.models.setup.setup_dockwindow.SetupDockWindow.remove`.
            * Setup removal (API): :meth:`pybpodgui_api.models.board.setup_base.SetupBase.remove`.
        """

        for index in range(len(self.sessions) - 1, -1, -1):
            self.sessions[index].remove()
        self.experiment -= self
        self.experiment.node.removeChild(self.node)

    def node_key_pressed_event(self, event):
        """
        Sets key events for:
            * Remove setup: :class:`SetupTreeNode.remove`

        :param event: key event
        """
        if event.key() == QtCore.Qt.Key_Delete:
            self.remove()

    @property
    def name(self):
        if hasattr(self, 'node'):
            return str(self.node.text(0))
        else:
            return SetupWindow.name.fget(self)

    @name.setter
    def name(self, value):
        SetupWindow.name.fset(self, value)
        if hasattr(self, 'node'):
            self.node.setText(0, value)

    @property
    def tree(self):
        return self.experiment.tree



#!/usr/bin/python
# -*- coding: utf-8 -*-

__version__ = "1.5.1"
__author__ = "Carlos Mao de Ferro, Ricardo Ribeiro, Luís Teixeira"
__credits__ = ["Carlos Mao de Ferro", "Ricardo Ribeiro", 'Luís Teixeira']
__license__ = "Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>"
__maintainer__ = ["Carlos Mao de Ferro", "Ricardo Ribeiro", 'Luís Teixeira']
__email__ = ["cajomferro@gmail.com", "ricardojvr@gmail.com", 'micboucinha@gmail.com']
__status__ = "Development"

import logging

from confapp import conf

logger = logging.getLogger(__name__)

conf += 'pyforms_generic_editor.settings'

# # load the user settings in case the file exists
# try:
# 	import pyforms_generic_editor_user_settings
#
# 	conf += pyforms_generic_editor_user_settings
# except Exception as err:
# 	logger.debug('No user_settings available', exc_info=True)



import logging

from pybpodgui_api.exceptions.wrong_subject_error import WrongSubjectConfigured
from .subject_base import SubjectBase

logger = logging.getLogger(__name__)


class SubjectCom(SubjectBase):

    def can_run_task(self):
        # if already running we should stop it (the run button changes to stop when pressed)
        if self.setup.status == self.setup.STATUS_RUNNING_TASK:
            self.setup.stop_task()
            self.setup.status = self.setup.STATUS_READY
            return False

        if self.setup is None or self.setup == 0:
            raise Exception('Please select a setup before proceeding.')

        if not (len(self.setup.subjects) == 1 and self in self.setup.subjects):
            raise WrongSubjectConfigured('The subject is not properly configured in the selected setup.')

        return True

    def run_task(self):
        if self.can_run_task():
            self.setup.run_task()



import os
import tempfile

import pyforms
from pyforms_gui.controls.control_checkbox import ControlCheckBox
from pyforms_gui.controls.control_textarea import ControlTextArea
from scipy.io import wavfile
import numpy as np
from AnyQt import QtGui
from AnyQt.QtWidgets import QFileDialog, QStatusBar
from confapp import conf
from pybpod_soundcard_module.utils.generate_sound import generate_sound, WindowConfiguration
from pyforms_gui.basewidget import BaseWidget
from pyforms_gui.controls.control_button import ControlButton
from pyforms_gui.controls.control_combo import ControlCombo
from pyforms_gui.controls.control_emptywidget import ControlEmptyWidget
from pyforms_gui.controls.control_label import ControlLabel
from pyforms_gui.controls.control_number import ControlNumber
from pyforms_gui.controls.control_text import ControlText

from .module_api import SoundCardModule, SampleRate, DataType


class SoundGenerationPanel(BaseWidget):

    def __init__(self, parent_win=None, win_flag=None):
        BaseWidget.__init__(self, 'Sound Generation panel', parent_win=parent_win, win_flag=win_flag)

        self._generated = False

        self._save_file_checkbox = ControlCheckBox('Write sound to file',
                                                   default=False,
                                                   changed_event=self.__save_file_checkbox_evt)

        self._filename = ControlText('Sound filename', '', changed_event=self.__filename_changed_evt, enabled=False)
        self._saveas_btn = ControlButton('Save As...', default=self.__prompt_save_file_evt, enabled=False)

        self._freq_label = ControlLabel('Frequency (Hz)', style='margin-left:0')
        self._freq_left = ControlNumber('Left channel', default=1000, minimum=0, maximum=20000)
        self._freq_right = ControlNumber('Right channel', default=1000, minimum=0, maximum=20000)
        self._duration = ControlNumber('Duration (s)', default=1, minimum=0, maximum=10000, decimals=2)

        self._sample_rate = ControlCombo('Sample rate')
        self._sample_rate.add_item('96 KHz', SampleRate._96000HZ)
        self._sample_rate.add_item('192 KHz', SampleRate._192000HZ)

        self._create_window = ControlCheckBox('Create window',
                                              default=True,
                                              changed_event=self.__create_window_checkbox_evt)

        self._left_window_duration = ControlNumber(default=100, minimum=0, maximum=20000)
        self._right_window_duration = ControlNumber(default=100, minimum=0, maximum=20000)
        self._left_apply_window_start = ControlCheckBox(default=True)
        self._right_apply_window_start = ControlCheckBox(default=True)
        self._left_apply_window_end = ControlCheckBox(default=True)
        self._right_apply_window_end = ControlCheckBox(default=True)

        self._left_window_functions = ControlCombo('Left channel window')
        self._left_window_functions.add_item('Hanning', 'Hanning')
        self._left_window_functions.add_item('Hamming', 'Hamming')
        self._left_window_functions.add_item('Blackman', 'Blackman')
        self._left_window_functions.add_item('Bartlett', 'Bartlett')

        self._right_window_functions = ControlCombo('Right channel window')
        self._right_window_functions.add_item('Hanning', 'Hanning')
        self._right_window_functions.add_item('Hamming', 'Hamming')
        self._right_window_functions.add_item('Blackman', 'Blackman')
        self._right_window_functions.add_item('Bartlett', 'Bartlett')

        self._gen_btn = ControlButton('Generate sound',
                                      default=self.__generate_sound_and_save)

        self._wave_int = []

        # Define the organization of the forms
        self.formset = [
            (
                ['_save_file_checkbox',
                    ('_filename', '_saveas_btn'),
                    ['_duration', '_sample_rate'],
                    ('h5:Frequency',  ['_freq_left', '_freq_right'])],
                '    ',
                ['_create_window',
                    ('', 'Left channel', 'Right channel'),
                    ('Duration (ms)', '_left_window_duration', '_right_window_duration'),
                    ('Apply window to start', '_left_apply_window_start', '_right_apply_window_start'),
                    ('Apply window to end', '_left_apply_window_end', '_right_apply_window_end'),
                    '_left_window_functions',
                    '_right_window_functions']
            ),
            '_gen_btn'
        ]

        self.set_margin(10)

    @property
    def sample_rate(self):
        return self._sample_rate.value

    @property
    def filename(self):
        return self._filename.value

    @property
    def wave_int(self):
        return self._wave_int

    @property
    def generated(self):
        return self._generated

    def sound_generated(self):
        pass

    def __create_window_checkbox_evt(self):
        # enable or disable the the window configuration according to the checkbox
        status = self._create_window.value
        self._left_window_duration.enabled = status
        self._left_apply_window_start.enabled = status
        self._left_apply_window_end.enabled = status
        self._left_window_functions.enabled = status
        self._right_window_duration.enabled = status
        self._right_apply_window_start.enabled = status
        self._right_apply_window_end.enabled = status
        self._right_window_functions.enabled = status

    def __filename_changed_evt(self):
        if not self._filename.value:
            self._gen_btn.enabled = False
        else:
            self._gen_btn.enabled = True

    def __prompt_save_file_evt(self):
        """
        Opens a window for the user to select where to save the Harp sound file (.bin extension by default)
        """
        self._filename.value, _ = QFileDialog.getSaveFileName()
        if self._filename.value:
            self._gen_btn.enabled = True
        else:
            self._gen_btn.enabled = False

    def __save_file_checkbox_evt(self):
        self._filename.enabled = self._saveas_btn.enabled = self._save_file_checkbox.value
        if self._filename.value:
            self._gen_btn.enabled = True
        else:
            self._gen_btn.enabled = not self._save_file_checkbox.value

    def __generate_sound_and_save(self):
        if self._save_file_checkbox.value and not self._filename.value:
            self.warning("Please select a destination file for the generated sound", "No sound file selected")
            return

        window_config = WindowConfiguration(left_duration=int(self._left_window_duration.value) / 1000,
                                            left_apply_window_start=self._left_apply_window_start.value,
                                            left_apply_window_end=self._left_apply_window_end.value,
                                            left_window_function=self._left_window_functions.value,
                                            right_duration=int(self._right_window_duration.value) / 1000,
                                            right_apply_window_start=self._right_apply_window_start.value,
                                            right_apply_window_end=self._right_apply_window_end.value,
                                            right_window_function=self._right_window_functions.value)

        self._wave_int = generate_sound(self._filename.value,
                                        # _sample_rate.value is an Enum so we need to get the value from it
                                        self._sample_rate.value.value,
                                        self._duration.value,
                                        int(self._freq_left.value),
                                        int(self._freq_right.value),
                                        window_config)

        self.sound_generated()
        self._generated = True


class LoadSoundPanel(BaseWidget):

    def __init__(self, parent_win=None, win_flag=None):
        BaseWidget.__init__(self, 'Load sound panel', parent_win=parent_win, win_flag=win_flag)

        self._loaded = False

        self._filename = ControlText('Sound file', '')
        self._read_btn = ControlButton('Browse', default=self.__prompt_read_file_evt)

        self._wave_int = []

        self.formset = [
            ('_filename', '_read_btn')
        ]

        self.set_margin(10)

    def __prompt_read_file_evt(self):
        """
        Opens a window for user to select where to save the sound .bin file
        """
        self._filename.value, _ = QFileDialog.getOpenFileName(caption='Choose sound file',
                                                              filter='Harp sound file(*.bin);;WAV(*.wav)')
        if self._filename.value:
            # assume that if the file extension ends .wav is a wave file, otherwise just try to read as a Harp sound
            if self._filename.value.endswith('.wav'):
                try:
                    fs, data = wavfile.read(self._filename.value)
                except Exception:
                    self.critical("Error while opening the WAV file. Please try again or try loading another file.")
                    return
                self._wave_int = np.array(data, dtype=np.int32)
            else:
                self._wave_int = np.fromfile(self._filename.value, dtype=np.int32)

            self.sound_loaded()
            self._loaded = True

    @property
    def wave_int(self):
        return self._wave_int

    @property
    def loaded(self):
        return self._loaded

    def sound_loaded(self):
        pass


class SoundCardModuleGUI(SoundCardModule, BaseWidget):
    TITLE = 'Sound Card module'

    def __init__(self, parent_win=None):
        BaseWidget.__init__(self, self.TITLE, parent_win=parent_win)

        self._msg_duration = 3000

        self._usb_port = ControlCombo('USB port', changed_event=self.__combo_usb_ports_changed_evt)
        self._refresh_usb_ports = ControlButton('',
                                                icon=QtGui.QIcon(conf.REFRESH_SMALL_ICON),
                                                default=self.__refresh_usb_ports_btn_pressed,
                                                helptext="Press here to refresh the list of available devices.")
        self._connect_btn = ControlButton('Connect', default=self.__connect_btn_pressed)

        # Send data
        self._send_btn = ControlButton('Send to sound card', default=self.__send_btn_pressed, enabled=False)
        self._index_to_send = ControlNumber('Index to send', default=2, minimum=2, maximum=32)
        self._user_metadata_send = ControlTextArea('User metadata (optional)')
        self._description_send = ControlTextArea('Description (optional)')

        self._send_panel = ControlEmptyWidget()
        self._send_panel.value = [self._index_to_send,
                                  self._user_metadata_send,
                                  self._description_send,
                                  self._send_btn]
        self._send_panel.setContentsMargins(10, 10, 10, 10)

        # Receive data
        self._index_to_read = ControlNumber('Index to read', default=2, minimum=2, maximum=32)
        self._read_all_checkbox = ControlCheckBox('Read all indexes',
                                                  default=False,
                                                  changed_event=self.__read_all_checkbox_evt)
        self._clear_folder_checkbox = ControlCheckBox('Clear destination folder', default=False)
        self._dest_folder = ControlText('Destination folder', '', changed_event=self.__folder_changed_evt)
        self._browse_btn = ControlButton('Browse', default=self.__prompt_browse_file_evt)
        self._read_btn = ControlButton('Read data', default=self.__read_btn_pressed, enabled=False)

        self._receive_panel = ControlEmptyWidget()
        self._receive_panel.value = [self._read_all_checkbox,
                                     self._index_to_read,
                                     self._clear_folder_checkbox,
                                     self._dest_folder,
                                     self._browse_btn,
                                     self._read_btn]

        self._receive_panel.setContentsMargins(10, 10, 10, 10)

        self._sound_generation = SoundGenerationPanel(parent_win=self)
        self._sound_generation.sound_generated = self._sound_generated_evt
        self._sound_gen_panel = ControlEmptyWidget()
        self._sound_gen_panel.value = self._sound_generation

        self._sound_load = LoadSoundPanel(parent_win=self)
        self._sound_load.sound_loaded = self._sound_loaded_evt
        self._sound_load_panel = ControlEmptyWidget()
        self._sound_load_panel.value = self._sound_load

        self._sound_card = SoundCardModule()
        self._wave_int = []

        self.formset = [
            ('_usb_port', '_refresh_usb_ports', '_connect_btn'),
            {
                'a:Generate sound': ['_sound_gen_panel'],
                'b:Load sound from disk': ['_sound_load_panel'],
            },
            ' ',
            {
                'a:Send data': ['_send_panel'],
                'b:Receive data': ['_receive_panel']
            }
        ]

        self._fill_usb_ports()

        self.set_margin(10)

        self._status_bar = QStatusBar(parent=self)

    def init_form(self):
        super(SoundCardModuleGUI, self).init_form()
        self.layout().addWidget(self._status_bar)

    def _fill_usb_ports(self):
        self._usb_port.add_item('', '')
        if self._sound_card:
            usb_devices = self._sound_card.devices
            for n, item in enumerate(usb_devices):
                if item.port_number:
                    item_str = item.product + ' {n} (port={port})'.format(n=n, port=item.port_number)
                else:
                    item_str = item.product + ' {n}'.format(n=n)

                self._usb_port.add_item(item_str, item)

    def _sound_generated_evt(self):
        if self._sound_generation.filename:
            self._status_bar.showMessage("Sound generated successfully and saved to '{file}'.".format(
                file=self._sound_generation.filename),
                self._msg_duration)
        else:
            self._status_bar.showMessage("Sound generated successfully.", self._msg_duration)
        if not self._connect_btn.enabled:
            self._send_btn.enabled = True

    def _sound_loaded_evt(self):
        self._status_bar.showMessage("Sound loaded successfully from disk.", self._msg_duration)
        if not self._connect_btn.enabled:
            self._send_btn.enabled = True

    def __read_all_checkbox_evt(self):
        self._index_to_read.enabled = not self._read_all_checkbox.value

    def __read_btn_pressed(self):
        if self._sound_card and not self._sound_card.connected:
            self.warning("Please connect to the sound card before proceeding.", "Not connected to the sound card")
            return

        # check if read all or read a specific index
        index_to_read = None if self._read_all_checkbox.value is True else int(self._index_to_read.value)

        # check if the folder exists
        if not os.path.isdir(self._dest_folder.value):
            self.critical("Folder doesn't exist. Please correct the path to an existing folder and try again",
                          "Path not found")
            return

        # read sound
        self._sound_card.read_sounds(self._dest_folder.value, index_to_read, self._clear_folder_checkbox.value)
        self._status_bar.showMessage("Data read successfully from the sound card", self._msg_duration)

    def __combo_usb_ports_changed_evt(self):
        if self._sound_card:
            self._sound_card.close()
        self._connect_btn.enabled = True
        self._send_btn.enabled = False

    def __refresh_usb_ports_btn_pressed(self):
        tmp = self._usb_port.value
        self._usb_port.clear()
        self._fill_usb_ports()
        self._usb_port.value = tmp

    def __folder_changed_evt(self):
        if not self._dest_folder.value:
            self._read_btn.enabled = False
        else:
            self._read_btn.enabled = True

    def __prompt_browse_file_evt(self):
        self._dest_folder.value = QFileDialog.getExistingDirectory()

        if self._dest_folder.value:
            self._read_btn.enabled = True
        else:
            self._read_btn.enabled = False

    def __connect_btn_pressed(self):
        if not self._usb_port.value:
            self.warning("Please select a USB connected device before proceeding.", "No USB device selected")
            return

        if self._sound_card is not None:
            self._sound_card.close()

        self._sound_card = SoundCardModule(device=self._usb_port.value)

        # update visual elements
        self._connect_btn.enabled = False

        if self._sound_generation.generated or self._sound_load.loaded:
            self._send_btn.enabled = True

    def __send_btn_pressed(self):
        if self._sound_card and not self._sound_card.connected:
            self.warning("Please connect to the sound card before proceeding",
                         "No connection to the sound card established.")
            return

        if (self._sound_generation.generated or self._sound_load.loaded) and self._sound_card.connected:
            wave_int = self._sound_generation.wave_int if self._sound_generation.generated else self._sound_load.wave_int

            # process Filename, metadata info and description data (optional parameters)
            temp_dir = tempfile.gettempdir()
            usr_metadata_file = None
            if self._user_metadata_send:
                usr_metadata_file = os.path.join(temp_dir, 'user_metadata.bin')
                with open(usr_metadata_file, 'w') as f:
                    f.write(self._user_metadata_send.value)

            description_file = None
            if self._description_send:
                description_file = os.path.join(temp_dir, 'description.bin')
                with open(description_file, 'w') as f:
                    f.write(self._description_send.value)

            self._sound_card.send_sound(wave_int,
                                        int(self._index_to_send.value),
                                        self._sound_generation.sample_rate,
                                        DataType.INT32,
                                        os.path.basename(self._sound_generation.filename) if self._sound_generation.filename else "Sound{id:02d}".format(id=int(self._index_to_send.value)),
                                        usr_metadata_file,
                                        description_file
                                        )
            self._status_bar.showMessage("Sound sent successfully to the sound card.", self._msg_duration)

            # delete the temp files
            if usr_metadata_file is not None:
                os.unlink(usr_metadata_file)
            if description_file is not None:
                os.unlink(description_file)


if __name__ == '__main__':
    pyforms.start_app(SoundCardModuleGUI, geometry=(0, 0, 600, 500))



__version__ = '0.1.5'

from pybpod_gui_plugin_emulator.emulator_gui import EmulatorGUI

from confapp import conf

conf += 'pybpod_gui_plugin_emulator.settings'
conf += 'pybpod_gui_plugin_emulator.resources'



__version__ 	= "1.1.2"
__author__ 		= ['Sergio Copeto']
__credits__ 	= ["Sergio Copeto"]
__license__ 	= "MIT"
__maintainer__ 	= ['Sergio Copeto']
__email__ 		= ['sergio.copeto']
__status__ 		= "Development"

from confapp import conf

conf += 'pybpod_alyx_module.settings'

from pybpod_alyx_module.module import Alyx as BpodModule


#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os


ROTARYENCODER_PLUGIN_ICON = os.path.join( os.path.dirname(__file__), 'resources', 'rotary.png' )


ROTARYENCODER_PLUGIN_WINDOW_SIZE = 300, 500




import os
import shutil


def copy_directory(src, dest):
    '''
    Copies a directory's contents (including subfolders) to a destination directory.
    .. note:: Existing files in the destination folder will NOT be copied.

    :param src: The source directory
    :param dest: The destination directory
    :return:
    '''
    for src_dir, dirs, files in os.walk(src):
        dst_dir = src_dir.replace(src, dest)
        if not os.path.exists(dst_dir):
            os.mkdir(dst_dir)
        for file_ in files:
            src_file = os.path.join(src_dir, file_)
            shutil.copy(src_file, dst_dir)



import io
import sys

from pybpodapi.com.messaging.stderr import StderrMessage


class StderrBuffer(io.StringIO):
    def __init__(self, session):
        super().__init__()
        self.session = session

    def write(self, msg):
        self.session += StderrMessage(msg)

    def flush(self):
        sys.__stderr__.flush()



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os

from confapp import conf as settings
from pybpodapi.bpod.bpod_com_protocol_modules import BpodCOMProtocolModules
from pybpodapi.com.messaging.session_info import SessionInfo

import pybpodapi

from pybpodapi.session import Session


logger = logging.getLogger(__name__)


class BpodIO(BpodCOMProtocolModules):
    """
    Bpod I/O logic.
    """
    def __init__(self, serial_port=None, workspace_path=None, session_name=None, sync_channel=None, sync_mode=None):
        self.workspace_path = workspace_path if workspace_path is not None else settings.PYBPOD_SESSION_PATH
        self.session_name = session_name if session_name is not None else settings.PYBPOD_SESSION

        super(BpodIO, self).__init__(serial_port, sync_channel, sync_mode)

        self.session += SessionInfo("This is a PYBPOD file. Find more info at http://pybpod.readthedocs.io")
        self.session += SessionInfo(Session.INFO_BPODAPI_VERSION, pybpodapi.__version__)
        self.session += SessionInfo(Session.INFO_SESSION_NAME, self.session_name)
        self.session += SessionInfo(Session.INFO_SESSION_STARTED, self.session.start_timestamp)

    def create_session(self):
        return Session(os.path.join(self.workspace_path, self.session_name)+'.csv') if self.workspace_path else Session()

    def close(self):
        """
        Close connection with Bpod
        """
        super(BpodIO, self).close()

    def __del__(self):
        if hasattr(self, 'session') and self.session:
            del self._session

    @property
    def workspace_path(self):
        return self._workspace_path  # type: str

    @workspace_path.setter
    def workspace_path(self, value):
        self._workspace_path = value  # type: str

    @property
    def session_name(self):
        return self._session_name  # type: str

    @session_name.setter
    def session_name(self, value):
        self._session_name = value  # type: str



# !/usr/bin/python3
# -*- coding: utf-8 -*-

from confapp import conf
from pybpodgui_plugin.models.subject.subject_uibusy import SubjectUIBusy

Subject = type(
    'Subject',
    tuple(conf.GENERIC_EDITOR_PLUGINS_FINDER.find_class('models.subject.Subject') + [SubjectUIBusy]),
    {}
)



import os

EMULATOR_PLUGIN_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'develop_board.svg')
EMULATOR_LAUNCH_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'launch.svg')
EMULATOR_CHECKED_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'checkbox_checked.svg')
EMULATOR_UNCHECKED_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'checkbox_unchecked.svg')

EMULATOR_PLUGIN_WINDOW_SIZE = 300, 400



# !/usr/bin/python3
# -*- coding: utf-8 -*-

""" loggingbootstrap

"""

__version__ = "1.0.0"
__author__ = "Carlos Mão de Ferro"
__credits__ = ["Carlos Mão de Ferro", "Ricardo Ribeiro"]
__license__ = "Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>"
__maintainer__ = ["Carlos Mão de Ferro", "Ricardo Ribeiro"]
__email__ = ["cajomferro@gmail.com", "ricardojvr@gmail.com"]
__status__ = "Development"

import logging


def create_console_logger(log_name, console_handler_level):
    """
    Create logger with console handler
    :param log_name: Name of the logger to be used when logger is invoked
    :param console_handler_level: logger level for console handler
    """
    loggers_formatter = logging.Formatter(
        '%(asctime)s | %(levelname)s | %(process)d | %(name)s | %(funcName)s | %(message)s',
        datefmt='%d/%m/%Y %I:%M:%S')

    logger = logging.getLogger(log_name)
    logger.setLevel(logging.DEBUG)

    ch = logging.StreamHandler()
    ch.setLevel(console_handler_level)
    ch.setFormatter(loggers_formatter)
    logger.addHandler(ch)


def create_file_logger(log_name, log_filename, file_handler_level):
    """
    Create simple logger with console and file handlers
    :param log_name: Name of the logger to be used when logger is invoked
    :param log_filename: name of the filename where file handler will output
    :param file_handler_level: logger level for file handler
    """
    loggers_formatter = logging.Formatter(
        '%(asctime)s | %(levelname)s | %(name)s | %(funcName)s | %(message)s',
        datefmt='%d/%m/%Y %I:%M:%S')

    logger = logging.getLogger(log_name)
    logger.setLevel(logging.DEBUG)

    fh = logging.FileHandler(log_filename)
    fh.setLevel(file_handler_level)
    fh.setFormatter(loggers_formatter)
    logger.addHandler(fh)


def create_double_logger(log_name, console_handler_level, log_filename, file_handler_level):
    """
    Create simple logger with console and file handlers
    :param log_name: Name of the logger to be used when logger is invoked
    :param log_filename: name of the filename where file handler will output
    :param file_handler_level: logger level for file handler
    :param console_handler_level: logger level for console handler
    """

    create_console_logger(log_name, console_handler_level)
    create_file_logger(log_name, log_filename, file_handler_level)



from pybpod_alyx_module.alyxapi.data.get import Get
from pybpod_alyx_module.alyxapi.data.post import Post

class Data():
    def __init__(self, _apibase):
        self.apibase = _apibase
        self.get = Get(_apibase)
        self.post = Post(_apibase)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import logging

from confapp import conf

from AnyQt.QtWidgets import QFileDialog
from AnyQt.QtGui import QIcon

from pybpodgui_plugin.models.project.project_window import ProjectWindow
from pybpodgui_plugin.models.experiment import Experiment
from pybpodgui_plugin.models.board import Board
from pybpodgui_plugin.models.task import Task
from pybpodgui_plugin.models.subject import Subject
from pybpodgui_plugin.models.user import User

logger = logging.getLogger(__name__)


class ProjectTreeNode(ProjectWindow):

    def __init__(self, projects):
        ProjectWindow.__init__(self)

        self.projects = projects
        self.projects += self
        self._loggeduser = None

        self.create_treenode(self.tree)

    def create_treenode(self, tree):
        self.node = tree.create_child(self.name, icon=QIcon(conf.PROJECT_SMALL_ICON))
        self.node.window = self
        self.node.setExpanded(True)

        tree.add_popup_menu_option('Save', self.save, item=self.node, icon=QIcon(conf.SAVE_SMALL_ICON))
        tree.add_popup_menu_option('Save as', self.save_as, item=self.node, icon=QIcon(conf.SAVE_SMALL_ICON))
        tree.add_popup_menu_option('Close', self.close, item=self.node, icon=QIcon(conf.CLOSE_SMALL_ICON))

        self.experiments_node = tree.create_child('Experiments', parent=self.node,
                                                  icon=QIcon(conf.EXPERIMENTS_SMALL_ICON))
        self.experiments_node.window = self
        self.experiments_node.setExpanded(True)

        tree.add_popup_menu_option('Add experiment', self._add_experiment, item=self.experiments_node,
                                   icon=QIcon(conf.ADD_SMALL_ICON))

        self.subjects_node = tree.create_child('Subjects', parent=self.node, icon=QIcon(conf.SUBJECTS_SMALL_ICON))
        tree.add_popup_menu_option('Add subject', self.create_subject, item=self.subjects_node,
                                   icon=QIcon(conf.ADD_SMALL_ICON))
        self.subjects_node.window = self
        self.subjects_node.setExpanded(True)

        self.boards_node = tree.create_child('Bpod boards', parent=self.node, icon=QIcon(conf.BOARDS_SMALL_ICON))
        self.boards_node.window = self
        self.boards_node.setExpanded(True)

        tree.add_popup_menu_option('Add Bpod boards', self._add_board, item=self.boards_node,
                                   icon=QIcon(conf.ADD_SMALL_ICON))

        self.tasks_node = tree.create_child('Protocols', parent=self.node, icon=QIcon(conf.TASKS_SMALL_ICON))
        tree.add_popup_menu_option('Add protocol', self._add_task, item=self.tasks_node,
                                   icon=QIcon(conf.ADD_SMALL_ICON))
        self.tasks_node.window = self
        self.tasks_node.setExpanded(True)

        tree.add_popup_menu_option('Import protocol', self.import_task, item=self.tasks_node,
                                   icon=QIcon(conf.OPEN_SMALL_ICON))
        tree.add_popup_menu_option('Import protocol (with folder contents)',
                                   self.import_task_with_directory,
                                   item=self.tasks_node,
                                   icon=QIcon(conf.OPEN_SMALL_ICON))

        self.users_node = tree.create_child('Users', parent=self.node, icon=QIcon(conf.PERSONS_SMALL_ICON))
        self.users_node.window = self
        self.users_node.setExpanded(True)
        tree.add_popup_menu_option('Add User', self._add_user, item=self.users_node, icon=QIcon(conf.ADD_SMALL_ICON))

        return self.node

    def _add_user(self):
        entity = self.create_user()
        # entity.focus_name()

    def _add_experiment(self):
        entity = self.create_experiment()
        entity.focus_name()

    def _add_board(self):
        entity = self.create_board()
        entity.focus_name()

    def create_user(self):
        user = User(self)
        self.tree.setCurrentItem(user.node)
        return user

    def create_experiment(self):
        experiment = Experiment(self)
        self.tree.setCurrentItem(experiment.node)
        return experiment

    def create_board(self):
        board = Board(self)
        self.tree.setCurrentItem(board.node)
        return board

    def create_task(self):
        task = Task(self)
        self.tree.setCurrentItem(task.node)
        return task

    def create_subject(self):
        subject = Subject(self)
        self.tree.setCurrentItem(subject.node)
        return subject

    def close(self, silent=False):
        confirmation = True

        if not silent:
            reply = self.question('Are sure you want to close the project?', 'Close project')
            confirmation = (reply == 'yes')

        if confirmation:
            super(ProjectTreeNode, self).close(silent)
            tree = self.tree
            tree -= self.node

    def save_as(self):
        folder = QFileDialog.getExistingDirectory(self,
                                                  "Select a directory to save the project: {0}".format(self.name))
        if folder:
            folder = os.path.join(folder, self.name)
            self.save(str(folder))

    def load(self, project_path):
        super(ProjectTreeNode, self).load(project_path)
        self.tree.setCurrentItem(self.node)

    def _add_task(self):
        if self.path is None or len(self.path) == 0 or not self.is_saved():
            reply = self.warning('To create a new protocol you need to save the project first.', 'Project not saved yet')
        else:
            task = self.create_task()
            task.filepath = task.make_emptyfile()
            task.focus_name()

    def import_task(self, filepath=None, with_directory=False):
        """
        Import task file to project

        Qt5 change:
        https://www.reddit.com/r/learnpython/comments/2xhagb/pyqt5_trouble_with_openinggetting_the_name_of_the/

        :param filepath:
        :return:
        """
        if self.path is None or len(self.path) == 0 or not self.is_saved():
            self.warning('To import a protocol you need to save the project first.', 'Project not saved yet')
        else:
            if not filepath:
                filepath, _ = QFileDialog.getOpenFileName(self, 'OpenFile')

            if filepath:
                try:
                    return super(ProjectTreeNode, self).import_task(filepath, with_directory)
                except Exception as e:
                    self.warning(str(e), 'Import aborted')

        return None

    def import_task_with_directory(self, filepath=None):
        return self.import_task(filepath, True)

    def user_removed(self, value):
        if self._loggeduser is not None and value.name == self._loggeduser.name:
            self._loggeduser = None
            self.name = self.name  # let's see if it auto updates

    @property
    def name(self):
        # if hasattr(self, 'node'):
        #   return str(self.node.text(0))
        # else:
        name = ProjectWindow.name.fget(self)
        if len(name) == 0:
            name = "Untitled project {0}".format(len(self.projects.projects))
            ProjectWindow.name.fset(self, name)
            return name
        else:
            return ProjectWindow.name.fget(self)

    @name.setter
    def name(self, value):
        ProjectWindow.name.fset(self, value)
        if hasattr(self, 'node'):
            if self.loggeduser is not None:
                aux = self.name + ' (' + self._loggeduser.name + '@' + self._loggeduser.connection + ')'
                self.node.setText(0, aux)
            else:
                self.node.setText(0, value)

    @property
    def tree(self):
        return self.projects.tree

    @property
    def loggeduser(self):
        return self._loggeduser

    @loggeduser.setter
    def loggeduser(self, value):
        self._loggeduser = value
        aux = self.name + ' (' + self._loggeduser.name + '@' + self._loggeduser.connection + ')'
        self.node.setText(0, aux)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf

from AnyQt.QtGui import QIcon

from pyforms.controls import ControlTree

from pybpodgui_plugin_timeline.trials_plot_window import TrialsPlotWindow

logger = logging.getLogger(__name__)


class SessionTreeNode(object):

    def create_treenode(self, tree):
        """

        :param ControlTree tree: project tree

        :return: this session tree node
        """
        node = super(SessionTreeNode, self).create_treenode(tree)
        self.trialsplot_action = tree.add_popup_menu_option(
            'Bars graph',
            self.__open_trials_plot_plugin,
            item=self.node,
            icon=QIcon(conf.TIMELINE_PLUGIN_ICON)
        )
        return node

    def __open_trials_plot_plugin(self):
        if not hasattr(self, 'trialsplot_win'):
            self.trialsplot_win = TrialsPlotWindow(self)
            self.trialsplot_win.show()
            self.trialsplot_win.subwindow.resize(
                *conf.TIMELINE_PLUGIN_WINDOW_SIZE)
        else:
            self.trialsplot_win.show()

        self.trialsplot_action.setEnabled(False)

    def remove(self):
        if hasattr(self, 'trialsplot_win'):
            self.mainwindow.mdi_area -= self.trialsplot_win
        super(SessionTreeNode, self).remove()

    @property
    def name(self):
        return super(SessionTreeNode, self.__class__).name.fget(self)

    @name.setter
    def name(self, value):
        super(SessionTreeNode, self.__class__).name.fset(self, value)
        if hasattr(self, 'trialsplot_win'):
            self.trialsplot_win.title = value



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import shutil

from pybpodgui_api.models.experiment.experiment_base import ExperimentBase

logger = logging.getLogger(__name__)


class ExperimentIO(ExperimentBase):
    """
    Save and Load actions for Experiment
    """
    def __init__(self, project):
        super(ExperimentIO, self).__init__(project)

        # initial name. Used to track if the name was updated
        self.initial_name = None

    ##########################################################################
    #       FUNCTIONS                                                        #
    ##########################################################################

    def collect_data(self, data):
        data.update({'name': self.name})
        # data.update({'task': self.task.name if self.task else None})
        data.update({'setups': []})

        for setup in self.setups:
            data['setups'].append(setup.collect_data({}))

        return data

    def save(self):
        """
        Save experiment data on filesystem.

        :ivar dict parent_path: Project path.
        :return: Dictionary containing the experiment info to save.
        :rtype: dict
        """
        if not self.name:
            logger.warning("Skipping experiment without name")
        else:
            if self.initial_name is not None:
                initial_path = os.path.join(self.project.path, 'experiments', self.initial_name)

                if initial_path != self.path:
                    shutil.move(initial_path, self.path)
                    # current_filepath = os.path.join(self.path, self.initial_name+'.json')
                    # future_filepath  = os.path.join(self.path, self.name+'.json')
                    # shutil.move( current_filepath, future_filepath )

            if not os.path.exists(self.path):
                os.makedirs(self.path)

            self.initial_name = self.name

            # save setups
            for setup in self.setups:
                setup.save()

            self.project.remove_non_existing_repositories(
                os.path.join(self.path, 'setups'),
                [setup.name for setup in self.setups]
            )

    def load(self, path):
        """
        Load experiment data from filesystem

        :ivar str experiment_path: Path of the experiment
        :ivar dict data: data object that contains all experiment info
        :return: Dictionary with loaded experiment info.
        """
        self.name = os.path.basename(path)
        # with open( os.path.join(self.path, self.name+'.json'), 'r' ) as stream:
        #     data = json.load(stream)
        # self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4

        self.initial_name = self.name

        setupspath = os.path.join(self.path, 'setups')
        if os.path.exists(setupspath):
            for name in os.listdir(setupspath):
                if os.path.isfile(os.path.join(setupspath, name)):
                    continue
                setup = self.create_setup()
                setup.load(os.path.join(setupspath, name))

    def __clean_setups_path(self, experiment_path):
        # remove from the setups directory the unused setup files
        setups_paths = [setup.path for setup in self.setups]
        for path in self.__list_all_setups_in_folder(experiment_path):
            if path not in setups_paths:
                shutil.rmtree(path) if os.path.isdir(path) else os.remove(path)

    def __generate_experiments_path(self, project_path):
        return os.path.join(project_path, 'experiments')

    def __generate_experiment_path(self, experiments_path):
        return os.path.join(experiments_path, self.name)

    def __list_all_setups_in_folder(self, experiment_path):
        search_4_dirs_path = os.path.join(experiment_path, 'setups')
        if not os.path.exists(search_4_dirs_path):
            return []
        return sorted([os.path.join(search_4_dirs_path, d) for d in os.listdir(search_4_dirs_path) if
                       os.path.isdir(os.path.join(search_4_dirs_path, d))])



import requests
import json
from confapp import conf

#self.apibase.addr + '/subjects' = conf.ALYX_PLUGIN_ADDRESS+'/subjects'

class Get():

    def __init__(self,_apibase):
        self.apibase = _apibase
        

    def allsubjects(self):
        result = requests.get(self.apibase.addr + '/subjects',headers=self.apibase.headers)
        if result.ok:
            result_data = result.json()
            return result_data
            
        return None

    
    def usersubjects(self, username):
        result = requests.get(self.apibase.addr + '/subjects?responsible_user=' + username, headers=self.apibase.headers)
        if result.ok:
            result_data = result.json()
            return result_data
            
        return None


    def bynickname(self, name):
        result = requests.get(self.apibase.addr + '/subjects'+'/'+name, headers=self.apibase.headers)
        if result.ok:
            result_data = result.json()
            return result_data

    def alive(self, _alive):
        _data = dict(alive= _alive)
        result = requests.get(self.apibase.addr + '/subjects',headers= self.apibase.headers,data = _data)
        if result.ok:
            
            print(result.json())

    def stock(self, _stock):
        _data = dict(stock= _stock)
        result = requests.get(self.apibase.addr + '/subjects',headers= self.apibase.headers,data = _data)
        if result.ok:
            
            print(result.json())
    
    def water_restircted(self, _water_restircted):
        _data = dict(water_restircted= _water_restircted)
        result = requests.get(self.apibase.addr + '/subjects',headers= self.apibase.headers,data = _data)
        if result.ok:
            
            print(result.json())

    def responsible_user(self, _responsible_user):
        _data = dict(responsible_user= _responsible_user)
        result = requests.get(self.apibase.addr + '/subjects',headers= self.apibase.headers,data = _data)
        rdata = []
        if result.ok:
            rd = result.json()
            for r in rd:
                if r['responsible_user'] == _responsible_user:
                    rdata.append(r)
        print(rdata)
        print(rd)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import logging

from confapp import conf

import pkgutil

from AnyQt.QtWidgets import qApp, QMessageBox
from AnyQt.QtGui import QIcon

from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlMdiArea

from pyforms_generic_editor.editor.generic_file_editor import GenericFileEditor

logger = logging.getLogger(__name__)


class SafeModeEditor(BaseWidget):
	def __init__(self):

		global conf
		conf += 'pyforms_generic_editor.resources'

		BaseWidget.__init__(self, conf.GENERIC_EDITOR_TITLE)

		self.mdi_area = ControlMdiArea()

		self.mainmenu = [
			{'File': [
				{'Exit': self._exit_app, 'icon': QIcon(conf.EXIT_SMALL_ICON)}
			]},
			{'Options': [
				{'Edit user settings': self._edit_user_settings, 'icon': QIcon(conf.USER_SETTINGS_ICON)}
			]},
			{'Help': [  # this option works for OSX also
				{'About QT': self._option_about_qt},
				{'About the application': self._option_about}
			]}
		]

		self._edit_user_settings()
		self._show_app_log()

		self.mdi_area.tileSubWindows()

	def init_form(self):
		error_message = conf.GENERIC_EDITOR_LOAD_EXCEPTION_TRACEBACK

		msg = QMessageBox()
		msg.setIcon(QMessageBox.Critical)

		msg.setText("Invalid settings file")
		msg.setInformativeText("Please correct user settings file.")
		msg.setWindowTitle("Invalid settings file")
		msg.setDetailedText(error_message)
		msg.setStandardButtons(QMessageBox.Ok)

		retval = msg.exec_()

		super(SafeModeEditor, self).init_form()

	def _option_about_qt(self):
		self.aboutQt("About Qt")

	def _option_about(self):
		msg = "About {0}\n".format(self.title)
		for module in self.dependencies:
			msg += "\n{0} version: {1}".format(str(module), module.__version__)
		for module in self.plugins_classes:
			parent_module = sys.modules[module.__module__]
			version = parent_module.__version__ if hasattr(parent_module, '__version__') else 'Undefined'
			msg += "\n{0}:\t\t{1}".format(str(module.__name__), version)
		self.about(msg, "About {0}".format(self.title))

	def _edit_user_settings(self):
		try:
			if not hasattr(self, '_user_settings_editor'):
				user_settings_path = pkgutil.get_loader("pyforms_generic_editor_user_settings").get_filename()
				self._user_settings_editor = GenericFileEditor(user_settings_path, "user settings editor")
				self._user_settings_editor.set_cursor_position(conf.GENERIC_EDITOR_LOAD_EXCEPTION_LINE - 3, 0)
			self.mdi_area += self._user_settings_editor
		except Exception as err:
			logger.warning("User settings file not found")
			self.warning("User settings file doesn't exist.", "Cannot edit settings file")

	def _show_app_log(self):
		try:
			if not hasattr(self, '_app_log_editor'):
				app_log_path = "app.log"
				self._app_log_editor = GenericFileEditor(app_log_path, "app.log", read_only=False)
				self._app_log_editor.editor._code_editor.setCursorPosition(
					self._app_log_editor.editor._code_editor.lines() - 15, 0)
			self.mdi_area += self._app_log_editor
		except FileNotFoundError as err:
			logger.warning("User settings file not found: %s", os.path.realpath(app_log_path))
			self.warning("Log file doesn't exist.", "Cannot show app log file")

	def _exit_app(self):
		"""
		Exit app option from the menu
		"""
		qApp.quit()



# !/usr/bin/python3
# -*- coding: utf-8 -*-

""" pybpod gui API

"""

import sys
import os
from confapp import conf

__version__ = "1.8.2"
__author__ = "Ricardo Jorge Vieira Ribeiro"
__credits__ = ["Ricardo Ribeiro", "Carlos Mão de Ferro", 'Luís Teixeira']
__license__ = "MIT"
__maintainer__ = ["Ricardo Ribeiro", "Carlos Mão de Ferro", 'Luís Teixeira']
__email__ = ["ricardojvr@gmail.com", "cajomferro@gmail.com", 'micboucinha@gmail.com']
__status__ = "Development"

# Load settings using pyforms library
# Include user settings in case the file exists

conf += 'pybpodgui_api.settings'

# try:
# 	import pycontrolapi.user_settings_api as user_settings
#
# 	conf += user_settings
# except:
# 	pass

# Workaround for pyInstaller and multiprocessing on Windows
# https://github.com/pyinstaller/pyinstaller/wiki/Recipe-Multiprocessing
# Module multiprocessing is organized differently in Python 3.4+
try:
    # Python 3.4+
    if sys.platform.startswith('win'):
        import multiprocessing.popen_spawn_win32 as forking
    else:
        import multiprocessing.popen_fork as forking
except ImportError:
    import multiprocessing.forking as forking

if sys.platform.startswith('win'):
    # First define a modified version of Popen.

    class _Popen(forking.Popen):
        def __init__(self, *args, **kw):
            if hasattr(sys, 'frozen'):
                # We have to set original _MEIPASS2 value from sys._MEIPASS
                # to get --onefile mode working.
                os.putenv('_MEIPASS2', sys._MEIPASS)
            try:
                super(_Popen, self).__init__(*args, **kw)
            finally:
                if hasattr(sys, 'frozen'):
                    # On some platforms (e.g. AIX) 'os.unsetenv()' is not
                    # available. In those cases we cannot delete the variable
                    # but only set it to the empty string. The bootloader
                    # can handle this case.
                    if hasattr(os, 'unsetenv'):
                        os.unsetenv('_MEIPASS2')
                    else:
                        os.putenv('_MEIPASS2', '')

    # Second override 'Popen' class with our modified version.
    forking.Popen = _Popen



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os

SETTINGS_PRIORITY = 80

# THESE SETTINGS ARE NEEDED FOR PYSETTINGS

SESSIONLOG_PLUGIN_ICON = os.path.join(os.path.dirname(__file__), 'resources', 'history.png')

SESSIONLOG_PLUGIN_WINDOW_SIZE 	= 700, 600
SESSIONLOG_PLUGIN_REFRESH_RATE 	= 1000



import logging
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import random
import traceback

from confapp import conf

from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlProgress
from pyforms.controls import ControlButton
from pyforms.controls import ControlCheckBox
from pyforms.controls import ControlList
from pyforms.controls import ControlBoundingSlider
from pyforms.controls import ControlMatplotlib
from AnyQt.QtWidgets import QApplication
from AnyQt.QtGui import QColor, QBrush
from AnyQt.QtCore import QTimer, QEventLoop, QAbstractTableModel, Qt, QSize, QVariant, pyqtSignal

#######################################################################
##### MESSAGES TYPES ##################################################
#######################################################################
from pybpodapi.com.messaging.error   import ErrorMessage
from pybpodapi.com.messaging.debug   import DebugMessage
from pybpodapi.com.messaging.stderr  import StderrMessage
from pybpodapi.com.messaging.stdout  import StdoutMessage
from pybpodapi.com.messaging.warning import WarningMessage
from pybpodapi.com.messaging.parser  import MessageParser

from pybpodapi.com.messaging.trial                  import Trial
from pybpodapi.com.messaging.end_trial              import EndTrial
from pybpodapi.com.messaging.event_occurrence       import EventOccurrence
from pybpodapi.com.messaging.state_occurrence       import StateOccurrence
from pybpodapi.com.messaging.softcode_occurrence    import SoftcodeOccurrence
from pybpodapi.com.messaging.event_resume           import EventResume
from pybpodapi.com.messaging.session_info           import SessionInfo
#######################################################################
#######################################################################
from pybpodapi.session import Session as APISession
from pybpodgui_api.models.session import Session

from matplotlib import colors as mcolors

class TrialTimeline(BaseWidget):

    COL_MSGTYPE   = 0
    COL_PCTIME    = 1
    COL_INITTIME  = 2
    COL_FINALTIME = 3
    COL_MSG       = 4
    COL_INFO      = 5

    def __init__(self, session : Session):
        BaseWidget.__init__(self, session.name)
        self.set_margin(5)

        self.session = session

        self._reload = ControlButton('Reload')
        self._graph  = ControlMatplotlib('Value')

        self._timer = QTimer()
        self._timer.timeout.connect(self.update)
        
        self._read = 0
        self._deltas = None

        self._last_trial_end = None
        
        self._states_dict = {}
        self._trials_list = []
        
        self.formset = [
            '_graph'            
        ]
        self.colors = list(mcolors.CSS4_COLORS.values())
        
        self._graph.on_draw = self.__on_draw_evt
        self._reload.value = self.__reload_evt

    
    
    def __reload_evt(self):
        if self._timer.isActive():
            self._timer.stop()
        else:
            self._timer.start(conf.TRIALTIMELINE_PLUGIN_REFRESH_RATE)

    
    def show(self, detached = False):
        if self.session.is_running and self.session.setup.detached:
            return
        
        # Prevent the call to be recursive because of the mdi_area
        if not detached:
            if hasattr(self, '_show_called'):
                BaseWidget.show(self)
                return
            self._show_called = True
            self.mainwindow.mdi_area += self
            del self._show_called
        else:
            BaseWidget.show(self)
        
        self._stop = False # flag used to close the gui in the middle of a loading
        if not self._stop and self.session.is_running:
            self._timer.start(conf.TRIALTIMELINE_PLUGIN_REFRESH_RATE)

        self.update()

    def hide(self):
        self._timer.stop()
        self._stop = True

    def read_data(self):
        
        res = self.session.data.query("TYPE in ['END-TRIAL', 'STATE'] or (TYPE == 'INFO' and MSG in ['SESSION-ENDED','TRIAL-BPOD-TIME'])")

        for index, msg in res.iterrows():
            if index<=self._read: continue
            
            if msg[self.COL_MSGTYPE] == EndTrial.MESSAGE_TYPE_ALIAS:
                
                if self._deltas is not None:
                    self._trials_list.append(self._deltas)
                self._deltas = {}
                
            elif msg[self.COL_MSGTYPE] == StateOccurrence.MESSAGE_TYPE_ALIAS:

                state = msg[self.COL_MSG]
                delta = float(msg[self.COL_FINALTIME]) - float(msg[self.COL_INITTIME])

                if state not in self._deltas:
                    # count, delta sum, min delta, max delta
                    self._deltas[state] = [delta]
                    self._states_dict[state] = True
                else:
                    self._deltas[state].append(delta)

            elif msg[self.COL_MSGTYPE] == SessionInfo.MESSAGE_TYPE_ALIAS and \
                 msg[self.COL_MSG]     == APISession.INFO_SESSION_ENDED:

                if self._deltas is not None:
                    self._trials_list.append(self._deltas)

            elif msg[self.COL_MSGTYPE] == SessionInfo.MESSAGE_TYPE_ALIAS and \
                 msg[self.COL_MSG] == APISession.INFO_TRIAL_BPODTIME:
                
                trial_start = msg[self.COL_INITTIME]
                trial_end   = msg[self.COL_FINALTIME]

                if self._last_trial_end is not None:
                    delta                             = float(trial_start)-float(self._last_trial_end)
                    self._deltas['Init lagging']      = [delta]
                    self._states_dict['Init lagging'] = True

                self._last_trial_end = trial_end            


            self._read = index
        

        


    def __on_draw_evt(self, figure):

        try:
            axes = figure.add_subplot(111)
            axes.clear()

            trials_labels  = []
            states_labels  = list(self._states_dict.keys())

            num_states = len(self._states_dict)
            num_trials = len(self._trials_list)
            data       = np.zeros( (num_states, num_trials) )
            errors     = np.zeros( (num_states, num_trials) )
            
            for i, states in enumerate(self._trials_list):
                trials_labels.append( 'Trial {0}'.format(i) )

                for j, state_label in enumerate(states_labels):

                    state_data = states.get(state_label, None)

                    if state_data is not None:
                        data[j][i]   = np.mean(state_data)
                        errors[j][i] = np.std(state_data)

            colors         = {}
            offset         = np.zeros( len(trials_labels) )
            trials_indexes = np.array(range(len(trials_labels)))

            for i, states_data in enumerate(data):
                if len(states_data)==0: continue

                state_label = states_labels[i]
                if state_label not in colors:
                    colors[state_label] = self.colors[len(colors)]

                axes.barh(
                    trials_indexes,
                    states_data,
                    height = 0.8,
                    color  = colors[state_label],
                    left   = offset,
                    label  = state_label,
                    yerr   = errors[i]
                )
                offset = offset + states_data
            
            axes.set_yticks(trials_indexes)
            axes.set_yticklabels(trials_labels)
            axes.set_ylabel('Trials')
            axes.set_xlabel('Time (sec)')
            axes.legend(loc="upper right")
            self._graph.repaint()
        except:
            self.critical( traceback.format_exc(), 'An error occurred')

                
    '''Takes care of all the session data and transforms it in a graph to be shown in the GUI'''
    def update(self):
        if not self.session.is_running:
            self._timer.stop()
        self.read_data()
        self._graph.draw()
    
    @property
    def mainwindow(self):
        return self.session.mainwindow

    
    @property
    def title(self):
        return BaseWidget.title.fget(self)
    
    
    @title.setter
    def title(self, value):
        BaseWidget.title.fset(self, 'Trial Timeline: {0}'.format(value))

    


import pyforms
import os
from pybpodapi.bpod import Bpod
from pybpodgui_api.exceptions.run_setup import RunSetupError
from pybpodgui_plugin.utils import make_lambda_func
from pyforms.basewidget import BaseWidget
from pyforms_gui.controls.control_button import ControlButton
from confapp import conf
from pyforms_gui.controls.control_label import ControlLabel
from pyforms_gui.controls.control_text import ControlText
from serial import SerialException


class EmulatorGUI(BaseWidget):
    """
    Main GUI for the Emulator module. This GUI window adapts automatically to the different Bpod versions that are
    connected to the computer to present correctly the number of Ports available as well as the connected modules to the
    Bpod modules ports.

    :param parent_win: The Setup object reference that this Emulator will be associated.
    """

    def __init__(self, parent_win=None):
        self.setup = parent_win
        self.started_correctly = False

        title = 'Emulator for setup: ' + self.setup.name

        BaseWidget.__init__(self, title, parent_win=parent_win)

        self.CHECKED_ICON = conf.EMULATOR_CHECKED_ICON
        self.UNCHECKED_ICON = conf.EMULATOR_UNCHECKED_ICON

        self._currentSetup = ControlLabel(self.setup.name)
        self._selectedBoard = ControlLabel(self.setup.board.name)
        self._selectedProtocol = ControlLabel(self.setup.task.name)

        self._run_task_btn = ControlButton('Run protocol',
                                           default=self.__run_protocol_btn_evt,
                                           checkable=True,
                                           helptext="When a task is running, you can skip all remaining trials by pressing this button. <br/> <b>NOTE:</b> This means that you will need to break the cycle in your task code when the run_state_machine method returns False.")
        self._kill_task_btn = ControlButton('Kill',
                                            default=self.__kill_btn_evt,
                                            style="background-color:rgb(255,0,0);font-weight:bold;",
                                            helptext="<b>NOTE:</b>This will exit the task process abruptly. The code you might have after the trial loop won't execute.")

        self._kill_task_btn.enabled = False

        self._stop_trial_btn = ControlButton('Skip trial',
                                             default=self.__stop_trial_btn_evt,
                                             enabled=False)
        self._pause_btn = ControlButton('Pause',
                                        default=self.__pause_btn_evt,
                                        checkable=True,
                                        enabled=False)

        try:
            bpod = Bpod(self.setup.board.serial_port)
        except SerialException:
            self.critical('No Bpod device connected, cannot continue until one is connected.', 'Bpod not connected')
            return
        except Exception:
            # NOTE: try again in case of the first connection attempt where we always get the utf-8 exception
            bpod = Bpod(self.setup.board.serial_port)

        number_ports = bpod.hardware.inputs.count('P')
        number_bnc = bpod.hardware.outputs.count('B')
        number_wire_in = bpod.hardware.inputs.count('W')
        number_wire_out = bpod.hardware.outputs.count('W')

        self._valve_buttons = []
        self._valve_label = ControlLabel("Valve")
        self._led_buttons = []
        self._led_label = ControlLabel("LED")
        self._poke_buttons = []
        self._poke_label = ControlLabel("Poke")

        for n in range(1, number_ports + 1):
            btn_valve = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)
            btn_led = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)
            btn_poke = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)

            btn_valve.value = make_lambda_func(self.__button_on_click_evt, btn=btn_valve)
            btn_led.value = make_lambda_func(self.__button_on_click_evt, btn=btn_led)
            btn_poke.value = make_lambda_func(self.__button_on_click_evt, btn=btn_poke)

            setattr(self, f'_btn_Valve{n}', btn_valve)
            setattr(self, f'_btn_PWM{n}', btn_led)
            setattr(self, f'_btn_Port{n}', btn_poke)
            self._valve_buttons.append(btn_valve)
            self._led_buttons.append(btn_led)
            self._poke_buttons.append(btn_poke)

        self._bnc_in_buttons = []
        self._bnc_in_label = ControlLabel("BNC In")
        self._bnc_out_buttons = []
        self._bnc_out_label = ControlLabel("BNC Out")

        for n in range(1, number_bnc + 1):
            btn_bnc_in = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)
            btn_bnc_out = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)

            btn_bnc_in.value = make_lambda_func(self.__button_on_click_evt, btn=btn_bnc_in)
            btn_bnc_out.value = make_lambda_func(self.__button_on_click_evt, btn=btn_bnc_out)

            setattr(self, f'_btn_BNC_in{n}', btn_bnc_in)
            setattr(self, f'_btn_BNC_out{n}', btn_bnc_out)
            self._bnc_in_buttons.append(btn_bnc_in)
            self._bnc_out_buttons.append(btn_bnc_out)

        self._wire_in_buttons = []
        self._wire_in_label = ControlLabel("Wire In")
        self._wire_out_buttons = []
        self._wire_out_label = ControlLabel("Wire Out")

        for n in range(1, number_wire_in + 1):
            btn_wire_in = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)
            btn_wire_in.value = make_lambda_func(self.__button_on_click_evt, btn=btn_wire_in)

            setattr(self, f'_btn_Wire_in{n}', btn_wire_in)
            self._wire_in_buttons.append(btn_wire_in)

        for n in range(1, number_wire_out + 1):
            btn_wire_out = ControlButton(str(n), icon=self.UNCHECKED_ICON, checkable=True)
            btn_wire_out.value = make_lambda_func(self.__button_on_click_evt, btn=btn_wire_out)

            setattr(self, f'_btn_Wire_out{n}', btn_wire_out)
            self._wire_out_buttons.append(btn_wire_out)

        self._modules_indexes_loaded = []

        for idx, mod in enumerate(bpod.modules):
            n = mod.serial_port
            self._modules_indexes_loaded.append(n)
            module_label = ControlLabel(f'{mod.name}')
            control_text_bytes_msg = ControlText()

            btn_send_msg_module = ControlButton(f'Send bytes')
            btn_send_msg_module.value = make_lambda_func(self.__send_msg_btn_evt,
                                                         btn=btn_send_msg_module,
                                                         control_text=control_text_bytes_msg)

            setattr(self, f'_module_label{n}', module_label)
            setattr(self, f'_control_text_bytes_msg{n}', control_text_bytes_msg)
            setattr(self, f'_btn_send_msg_module{n}', btn_send_msg_module)

        if bpod:
            bpod.close()

        self.formset = [
            ([('Current setup:', '_currentSetup'),
              ('Selected board:', '_selectedBoard'),
              ('Selected protocol:', '_selectedProtocol')],
             '',
             ['_run_task_btn', '_kill_task_btn', '_stop_trial_btn', '_pause_btn']),
            '',
            'Behaviour Ports',
            ('_valve_label', tuple([f'_btn_Valve{n.label}' for n in self._valve_buttons])),
            ('_led_label', tuple([f'_btn_PWM{n.label}' for n in self._led_buttons])),
            ('_poke_label', tuple([f'_btn_Port{n.label}' for n in self._poke_buttons])),
            '',
            'BNC',
            ('_bnc_in_label',
             tuple([f'_btn_BNC_in{n.label}' for n in self._bnc_in_buttons]),
             '_bnc_out_label',
             tuple([f'_btn_BNC_out{n.label}' for n in self._bnc_out_buttons])
             ),
            'Wire' if number_wire_in != 0 else '',
            ('_wire_in_label' if number_wire_in != 0 else '',
             tuple([f'_btn_Wire_in{n.label}' for n in self._wire_in_buttons]),
             '_wire_out_label' if number_wire_out != 0 else '',
             tuple([f'_btn_Wire_out{n.label}' for n in self._wire_out_buttons])
             ),
            '',
            'Send bytes to modules' if self._modules_indexes_loaded else '',
            [(f'_module_label{n}', f'_control_text_bytes_msg{n}', f'_btn_send_msg_module{n}') for n in self._modules_indexes_loaded]
        ]

        self.set_margin(10)
        self.started_correctly = True

    def show(self):
        """
        Overrides the BaseWidget implementation of the show method in order to update the textual information of the
        board and protocol used, in case of being updated in the main window after creation of this EmulatorGUI window.
        :return:
        """
        # update names on labels
        self._currentSetup.value = self.setup.name
        self._selectedBoard.value = self.setup.board.name
        self._selectedProtocol.value = self.setup.task.name

        self.init_form()
        super(BaseWidget, self).show()

    def update_task(self, task):
        """
        Method to update the task name
        :param task: The Task to be used to update the information in the UI, if available.
        :return:
        """
        self._selectedProtocol.value = task.name if task != 0 else ''

    def update_board(self, board):
        """
        Method to update the board name
        :param board: The Board to be used to update the information in the UI, if available.
        :return:
        """
        self._selectedBoard.value = board.name if board != 0 else ''

    def __send_msg_btn_evt(self, btn=None, control_text=None):
        # get message from textbox
        if btn is None or control_text is None:
            return
        module_index = btn.name[-1]
        message = f'message:{module_index}:{control_text.value}{os.linesep}'

        # send msg through stdin to bpod (we need to create a command first in the other side)
        self.setup.board.proc.stdin.write(message.encode('utf-8'))
        self.setup.board.proc.stdin.flush()

    def __button_on_click_evt(self, btn=None):
        if btn is None:
            return

        if self.setup.status is not self.setup.STATUS_RUNNING_TASK:
            return

        name = btn.name.split('_')
        port_name = name[2]
        is_pwm = port_name.startswith('PWM')
        is_valve = port_name.startswith('Valve')
        is_output = is_pwm or is_valve
        if len(name) > 3:
            port_number = name[3][-1]
            is_output = name[3].startswith('out')
        else:
            port_number = ''

        if btn.checked:
            val = 1
            if is_pwm:
                val = 255
            btn.icon = self.CHECKED_ICON
        else:
            val = 0
            btn.icon = self.UNCHECKED_ICON

        if is_output:
            message = f'trigger_output:{port_name}{port_number}:{val}{os.linesep}'
        else:
            message = f'trigger_input:{port_name}{port_number}:{val}{os.linesep}'

        self.setup.board.proc.stdin.write(message.encode('utf-8'))
        self.setup.board.proc.stdin.flush()

    def __run_protocol_btn_evt(self):
        try:
            if self.setup.status == self.setup.STATUS_RUNNING_TASK:
                self.setup.stop_task()
            elif self.setup.status == self.setup.STATUS_READY:
                self.setup.run_task()
        except RunSetupError as err:
            self.warning(str(err), "Warning")
        except Exception as err:
            self.alert(str(err), "Unexpected Error")

    def __kill_btn_evt(self):
        if self.setup.status == self.setup.STATUS_RUNNING_TASK:
            self.setup.kill_task()

    def __stop_trial_btn_evt(self):
        setup = self.setup
        if setup:
            setup._stop_trial_evt()
        else:
            self.critical("There isn't any setup selected. Please select one before continuing.", "No setup selected")

    def __pause_btn_evt(self):
        setup = self.setup
        if self._pause_btn.checked:
            self._pause_btn.label = 'Resume'
            setup.pause_trial()
        else:
            setup.resume_trial()
            self._pause_btn.label = 'Pause'


if __name__ == '__main__':
    pyforms.start_app(EmulatorGUI, geometry=(0, 0, 300, 300))



from pybpodapi.bpod_modules.bpod_module import BpodModule


class Alyx(BpodModule):
    
    @staticmethod
    def check_module_type(module_name):
        return module_name and module_name.startswith('Alyx')


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import datetime
import traceback
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText
from pyforms.controls import ControlList
from AnyQt.QtWidgets import QApplication
from pybpodgui_api.models.session import Session
from pybpodgui_api.exceptions.invalid_session import InvalidSessionError

logger = logging.getLogger(__name__)


class SessionWindow(Session, BaseWidget):
    """ ProjectWindow represents the project entity as a GUI window"""

    def __init__(self, setup=None):
        BaseWidget.__init__(self, 'Session')
        self.layout().setContentsMargins(5, 10, 5, 5)

        self._name = ControlText('Session')
        self._setup_name = ControlText('Setup')
        self._board_name = ControlText('Board')
        self._task_name = ControlText('Task')
        self._started = ControlText('Started on')
        self._ended = ControlText('Ended on')
        self._subjects = ControlList('Subjects')
        self._board_serial_port = ControlText('Serial port')
        self._variables = ControlList('Variables')

        Session.__init__(self, setup)

        self._formset = [
            '_name',
            '_started', '_ended',
            '_setup_name', '_task_name',
            '_board_name',
            '_board_serial_port',
            '_subjects',
            '_variables',
        ]

        self._subjects.readonly = True
        self._subjects.enabled = False
        self._variables.readonly = True
        self._variables.enabled = False
        self._setup_name.enabled = self._board_name.enabled = self._task_name.enabled = False
        self._board_serial_port.enabled = self._started.enabled = self._ended.enabled = False
        self._name.changed_event = self.__name_edited_evt

    def __add__(self, value):
        QApplication.processEvents()
        return super(SessionWindow, self).__add__(value)

    def __name_edited_evt(self):
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._name.value

    def load(self, path):
        try:
            Session.load(self, path)
        except InvalidSessionError as err:
            logger.warning(str(err))
        except:
            self.critical(
                'An error occurred when trying to load the info for session [{0}].\n\n{1}'.format(self.name, traceback.format_exc()),
                'Error loading the session')

    def remove(self):
        self.setup -= self
        for s in self.subjects:
            _, uuid = eval(s)
            sub = self.project.find_subject_by_id(uuid)
            if sub is not None:
                sub -= self

    def load_contents(self):
        try:
            if self.data is None and not self.is_running:
                super(SessionWindow, self).load_contents()
        except FileNotFoundError:
            logger.warning("Error when trying to load the session content.")

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def name(self):
        return self._name.value

    @name.setter
    def name(self, value):
        self._update_name = True  # Flag to avoid recurse calls when editing the name text field
        self._name.value = value
        self._update_name = False
        self.title = "{0}: {1}".format(self.setup.name, value)

    @property
    def setup_name(self):
        return self._setup_name.value

    @setup_name.setter
    def setup_name(self, value):
        self._setup_name.value = value

    @property
    def board_name(self):
        return self._board_name.value

    @board_name.setter
    def board_name(self, value):
        self._board_name.value = value

    @property
    def task_name(self):
        return self._task_name.value

    @task_name.setter
    def task_name(self, value):
        self._task_name.value = value

    @property
    def variables(self):
        return self._variables.value

    @variables.setter
    def variables(self, value):
        self._variables.value = value

    @property
    def board_serial_port(self):
        return self._board_serial_port.value

    @board_serial_port.setter
    def board_serial_port(self, value):
        self._board_serial_port.value = value

    @property
    def started(self):
        return datetime.datetime.strptime(self._started.value, '%Y/%m/%d %H:%M:%S') if len(
            self._started.value) > 0 else None

    @started.setter
    def started(self, value):
        self._started.value = value.strftime('%Y/%m/%d %H:%M:%S') if value else None

    @property
    def ended(self):
        return datetime.datetime.strptime(self._ended.value, '%Y/%m/%d %H:%M:%S') if len(
            self._ended.value) > 0 else None

    @ended.setter
    def ended(self, value):
        self._ended.value = value.strftime('%Y/%m/%d %H:%M:%S') if value else None

    @property
    def subjects(self):
        return [v[0] for v in self._subjects.value]

    @subjects.setter
    def subjects(self, value):
        self._subjects.value = [[v] for v in value]



import array
import math
import time
import traceback

import numpy as np
from enum import Enum, IntEnum
from aenum import auto
import os
import collections
import usb.core
import usb.util
from usb.backend import libusb1 as libusb

class SampleRate(IntEnum):
    """
    Enumeration for the Sample rate of the sounds in the Sound Card
    """

    #: 96KHz sample rate
    _96000HZ = 96000
    #: 192KHz sample rate
    _192000HZ = 192000


class DataType(IntEnum):
    """
    Type of the data to be send to the Sound Card
    """

    #: Integer 32 bits
    INT32 = 0,

    #: Single precision float
    FLOAT32 = 1


class SoundCardErrorCode(Enum):
    OK = 0,
    BAD_USER_INPUT = -1,

    HARP_SOUND_CARD_NOT_DETECTED = -1000,
    NOT_ABLE_TO_SEND_METADATA = auto(),
    NOT_ABLE_TO_READ_METADATA_COMMAND_REPLY = auto(),
    METADATA_COMMAND_REPLY_NOT_CORRECT = auto(),
    NOT_ABLE_TO_SEND_DATA = auto(),
    NOT_ABLE_TO_READ_DATA_COMMAND_REPLY = auto(),
    DATA_COMMAND_REPLY_NOT_CORRECT = auto(),
    NOT_ABLE_TO_SEND_READ_METADATA = auto(),
    NOT_ABLE_TO_READ_READ_METADATA_COMMAND_REPLY = auto(),
    READ_METADATA_COMMAND_REPLY_NOT_CORRECT = auto(),

    BAD_SOUND_INDEX = -1020,
    BAD_SOUND_LENGTH = auto(),
    BAD_SAMPLE_RATE = auto(),
    BAD_DATA_TYPE = auto(),
    DATA_TYPE_DO_NOT_MATCH = auto(),
    BAD_DATA_INDEX = auto(),

    PRODUCING_SOUND = -1030,
    STARTED_PRODUCING_SOUND = auto(),

    NOT_ABLE_TO_OPEN_FILE = -1040


class SoundMetadata(object):

    def __init__(self, sound_index, sound_length, sample_rate, data_type):
        """

        :param self:
        :param sound_index: Sound index in the soundcard (2 -> 31 since 0 and 1 are reserved)
        :param sound_length: Sound length in number of samples
        :param sample_rate: Sample rate
        :param data_type: 0 for Int32 and 1 for Float32 (not available right now)
        """
        self._sound_index = sound_index
        self._sound_length = sound_length
        self._sample_rate = sample_rate
        self._data_type = data_type

    def check_data(self):
        if self._sound_index < 2 or self._sound_index > 32:
            return SoundCardErrorCode.BAD_SOUND_INDEX

        if self._sound_length < 16:
            return SoundCardErrorCode.BAD_SOUND_LENGTH

        if self._sample_rate is not SampleRate._96000HZ and self._sample_rate is not SampleRate._192000HZ:
            return SoundCardErrorCode.BAD_SAMPLE_RATE

        if self._data_type is not DataType.INT32 and self._data_type is not DataType.FLOAT32:
            return SoundCardErrorCode.BAD_DATA_TYPE

        if self._sound_index == 0 and self._data_type is not DataType.FLOAT32:
            return SoundCardErrorCode.DATA_TYPE_DO_NOT_MATCH

        if self._sound_index == 1 and self._data_type is not DataType.FLOAT32:
            return SoundCardErrorCode.DATA_TYPE_DO_NOT_MATCH

        if self._sound_index > 1 and self._data_type is not DataType.INT32:
            return SoundCardErrorCode.DATA_TYPE_DO_NOT_MATCH

        return SoundCardErrorCode.OK

    def as_array(self):
        return np.array([self._sound_index, self._sound_length, self._sample_rate, self._data_type], dtype=np.int32)


class SoundCardModule(object):
    """
    Provides access to the Harp Sound Card. It allows to send and read the sounds in the Sound Card, through a normal
    USB connection.
    """

    def __init__(self, device=None):
        """
        If a libUSB's device is given, it will try to open it. If none is given it will try to connect to the first Sound Card that is connected to the computer.

        :param device: (Optional) libUSB device to use. If nothing is passed, it will try to connect automatically.
        """
        self._backend = libusb.get_backend()
        try:
            self._devices = list(usb.core.find(backend=self._backend, idVendor=0x04d8, idProduct=0xee6a, find_all=True))
        except OSError as e:
            pass
        
        self._dev = self._devices[0] if self._devices else None
        self._cfg = None
        self._port = None
        self._connected = False

        self.open(self._dev if device is None else device)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def open(self, device=None):
        """
        Opens the connection to the Sound Card. If no device is given, it will try to connect to the first Sound Card that is connected to the computer.

        :param device: (Optional) Already initialized libUSB's device to use.
        """
        if device is None:
            self._backend = libusb.get_backend()
            try:
                self._dev = usb.core.find(backend=self._backend, idVendor=0x04d8, idProduct=0xee6a)
            except OSError as e:
                self._dev = None
                pass
        else:
            self._dev = device

        if self._dev is None:
            print(
                "Unable to connect to the Sound Card through the USB port. You will be unable to send and receive sounds.")
        else:
            # set the active configuration. With no arguments, the first configuration will be the active one
            # note: some devices reset when setting an already selected configuration so we should check for it before
            self._cfg = self._dev.get_active_configuration()
            if self._cfg is None or self._cfg.bConfigurationValue != 1:
                self._dev.set_configuration(1)

        self._connected = True if self._dev else False

    @property
    def devices(self):
        return self._devices

    @property
    def connected(self):
        return self._connected

    def close(self):
        """
        Closes the connection with the Sound Card. It will close USB connection (to read and save sounds)
        """
        if self._dev:
            usb.util.dispose_resources(self._dev)

    def reset(self):
        """
        Resets the device, waits 700ms and tries to connect again so that the current instance of the SoundCard object can still be used.

        .. note:: Necessary at the moment after sending a sound.
        """
        if not self._dev:
            raise Exception("Sound card might not be connected. Please connect it before any operation.")

        # Reset command length:    'c' 'm' 'd' '0x88' + 'f'
        reset_cmd = [ord('c'), ord('m'), ord('d'), 0x88, ord('f')]
        # cmd = 'cmd' + chr(0x88) + 'f'
        wrt = self._dev.write(1, reset_cmd, 100)
        assert wrt == len(reset_cmd)

        time.sleep(700.0 / 1000.0)
        self.open()

    def read_sounds(self, output_folder=None, sound_index=None, clean_dst_folder=True):
        """
        Reads sounds from the sound card.

        .. note:: by default, it will clear the destination folder of all data. It will also write by default to a
                  "from_soundcard" folder in the working directory if none is given.

        :param output_folder: Destination folder's path.
        :param sound_index: If a sound_index is given, it will get only that sound, if nothing is passed it will gather all sounds from all indexes.
        :param clean_dst_folder: Flag that defines if the method should clean the destination folder or not

        """
        if not self._dev:
            raise Exception("Sound card might not be connected. Please connect it before any operation.")

        # admit that if the output_folder is None, write inside a 'from_soundcard' folder in the current directory
        if not output_folder:
            output_folder = os.path.join(os.getcwd(), 'from_soundcard')
            if not os.path.isdir(output_folder):
                os.makedirs(output_folder)
        else:
            # create folder if it doesn't exists
            if not os.path.exists(output_folder):
                os.makedirs(output_folder)

            if clean_dst_folder:
                for file in os.listdir(output_folder):
                    file_path = os.path.join(output_folder, file)
                    try:
                        if os.path.isfile(file_path):
                            os.unlink(file_path)
                    except Exception as e:
                        # probably a permissions error while deleting, ignore and try the next one
                        print("Error occurred when deleting file '{file_path}'. Ignoring error and continuing.".format(file_path=file_path))
                        continue

        if sound_index is None:
            for i in range(2, 32):
                self._from_soundcard(output_folder, i)
        else:
            self._from_soundcard(output_folder, sound_index)

        print("All files read!")

    def send_sound(self, wave_int, sound_index, sample_rate, data_type, 
                   sound_filename=None, metadata_filename=None, 
                   description_filename=None):
        """
            This method will send the sound to the Harp Sound Card as a byte array (int8)

            :param wave_int: NumPy array as int32 that represents the sound data
            :param sound_index:  The destination index in the Sound Card (>=2 and <= 32)
            :param sample_rate: The SampleRate enum value for either 96KHz or 192KHz
            :param data_type: The DataType enum value for either Int32 or Float32 (not implemented yet in the hardware)
            :param sound_filename: The name of the sound filename to be saved with the sound in the board (str)
            :param metadata_filename: The name of the metadata filename to be saved with the sound in the board (str)
            :param description_filename: The name of the description filename to be saved with the sound in the board (str)
        """
        self._to_soundcard(wave_int, sound_index, sample_rate, data_type, sound_filename, metadata_filename,
                          description_filename)

    def _from_soundcard(self, output_folder=None, sound_index=None):
        """
        Reads sounds from the sound card.

        :param output_folder: Destination folder's path.
        :param sound_index: If a sound_index is given, it will get only that sound, if nothing is passed it will gather all sounds from all indexes.

        """
        if not self._dev:
            raise Exception("Sound card might not be connected. Please connect it before any operation.")

        if sound_index is None or sound_index < 2 or sound_index > 31:
            raise Exception("sound_index must have a value between 2 and 31")

        metadata = self.__get_metadata_from_device(sound_index)

        if metadata is None:
            raise Exception('SoundCardModule: Error while getting metadata from device')

        # define prefix
        prefix = 'i'
        if sound_index < 9:
            prefix += '0' + str(sound_index) + '_'
        else:
            prefix += str(sound_index) + '_'

        sound_filename = metadata.sound_filename.decode('utf-8')
        metadata_filename = metadata.metadata_filename.decode('utf-8') if metadata.metadata_filename else None
        description_filename = metadata.description_filename.decode(
            'utf-8') if metadata.description_filename else None

        if prefix not in sound_filename:
            sound_filename = prefix + sound_filename
        if metadata_filename and prefix not in metadata_filename:
            metadata_filename = prefix + metadata_filename
        if description_filename and prefix not in description_filename:
            description_filename = prefix + description_filename

        if metadata.has_sound:
            with open(os.path.join(output_folder, sound_filename), 'w', encoding='utf8') as f:
                # TODO: read the sound so we can write it here
                f.write('TODO')

        if metadata.has_metadata:
            with open(os.path.join(output_folder, metadata_filename), 'wb') as f:
                # clean the zeros at the end
                f.write(metadata.metadata_array.tobytes().strip(b'\0'))

        if metadata.has_description:
            with open(os.path.join(output_folder, description_filename), 'wb') as f:
                f.write(metadata.description.tobytes().strip(b'\0'))

        # create summary info file
        if metadata.has_sound:
            with open(os.path.join(output_folder, sound_filename + '.metadata.txt'), 'w') as f:
                f.write('SOUND_INDEX = ' + str(sound_index))

                used_pos = math.ceil(metadata.sound_length / (33554432.0 * 2.0 / 32.0)) - 1
                if used_pos > 0:
                    f.write(", ")
                f.write(", ".join(str(sound_index + idx + 1) for idx in range(used_pos)))

                f.write("\n")

                f.write("TOTAL_SAMPLES = " + str(metadata.sound_length) + "\n")
                f.write(
                    "TOTAL_LENGTH_MS = " + str(int(metadata.sound_length / 2 / metadata.sample_rate * 1000)) + "\n")
                f.write("SAMPLE_RATE = " + str(metadata.sample_rate) + "\n")

                if metadata.data_type == 0:
                    f.write("DATA_TYPE = Int32\n")
                else:
                    f.write("DATA_TYPE = Float32\n")

                f.write("SOUND_FILENAME = " + sound_filename + "\n")

                if metadata.has_metadata:
                    f.write("USER_METADATA_FILENAME = " + metadata_filename + "\n")
                if metadata.has_description:
                    f.write("USER_DESCRIPTION_FILENAME = " + description_filename + "\n")

    def _to_soundcard(self, wave_int, sound_index, sample_rate, data_type,
                      sound_filename=None, metadata_filename=None, 
                      description_filename=None):
        """
            This method will send the sound to the Harp Sound Card as a byte array (int8)

            :param wave_int: NumPy array as int32 that represents the sound data
            :param sound_index:  The destination index in the Sound Card (>=2 and <= 32)
            :param sample_rate: The SampleRate enum value for either 96KHz or 192KHz
            :param data_type: The DataType enum value for either Int32 or Float32 (not implemented yet in the hardware)
            :param sound_filename: The name of the sound filename to be saved with the sound in the board (str)
            :param metadata_filename: The name of the metadata filename to be saved with the sound in the board (str)
            :param description_filename: The name of the description filename to be saved with the sound in the board (str)
        """
        # confirm that the dev exists and is ready
        if not self._dev:
            raise EnvironmentError(
                'Sound card not initialized. Please call the initialize method before any operation.')

        int32_size = np.dtype(np.int32).itemsize
        # work with a int8 view of the wave_int (which is int32)
        wave_int8 = wave_int.view(np.int8)

        # get number of commands to send
        sound_file_size_in_samples = len(wave_int8) // 4
        commands_to_send = int(sound_file_size_in_samples * 4 // 32768 + (
            1 if ((sound_file_size_in_samples * 4) % 32768) != 0 else 0))

        # Metadata command length: 'c' 'm' 'd' '0x80' + random + metadata  + 32768 + 2048 + 'f'
        metadata_cmd_header_size = 4 + int32_size + (4 * int32_size)
        metadata_cmd = np.zeros(metadata_cmd_header_size + 32768 + 2048 + 1, dtype=np.int8)

        metadata_cmd[0] = ord('c')
        metadata_cmd[1] = ord('m')
        metadata_cmd[2] = ord('d')
        metadata_cmd[3] = 0x80
        metadata_cmd[-1] = ord('f')

        rand_val = np.random.randint(-32768, 32768, size=1, dtype=np.int32)
        # copy that random data
        metadata_cmd[4: 4 + int32_size] = rand_val.view(np.int8)

        # create metadata info and add it to the metadata_cmd
        metadata = SoundMetadata(sound_index, sound_file_size_in_samples, sample_rate, data_type)
        if metadata.check_data() is not SoundCardErrorCode.OK:
            print("Input data incorrect, please correct it before proceeding.")
            return

        metadata_cmd[8: 8 + (4 * int32_size)] = metadata.as_array().view(np.int8)

        # add first data block of data to the metadata_cmd
        metadata_cmd_data_index = metadata_cmd_header_size
        metadata_cmd[metadata_cmd_data_index: metadata_cmd_data_index + 32768] = wave_int8[0: 32768]

        # prepare user_metadata
        # [0:169]     sound_filename
        # [170:339]   metadata_filename
        # [340:511]   description_filename
        # [512:1535]  metadata_filename content
        # [1536:2047] description_filename content
        user_metadata = np.zeros(2048, dtype=np.int8)
        user_metadata_index = metadata_cmd_data_index + 32768

        if sound_filename:
            tmp = bytearray()
            tmp.extend(map(ord, os.path.basename(sound_filename)))
            tmp_size = len(tmp) if len(tmp) < 169 else 169
            user_metadata[0:tmp_size] = tmp[0:tmp_size]

        if metadata_filename:
            tmp = bytearray()
            tmp.extend(map(ord, os.path.basename(metadata_filename)))
            tmp_size = len(tmp) if len(tmp) < 169 else 169
            user_metadata[170: 170 + tmp_size] = tmp[0:tmp_size]

            # get file contents, truncate data if required
            try:
                with open(metadata_filename, 'r', encoding='utf8') as f:
                    text = f.read()
                    text_tmp = bytearray()
                    text_tmp.extend(map(ord, text))
                    data_tmp = np.array(text_tmp)
                    data = data_tmp.view(np.int8)
                    data_size = len(data) if len(data) < 1023 else 1023
                    user_metadata[512: 512 + data_size] = data[0: data_size]
            except OSError as e:
                # TODO: should be a stronger error
                print("Error opening metadata file.")

        if description_filename:
            tmp = bytearray()
            tmp.extend(map(ord, os.path.basename(description_filename)))
            tmp_size = len(tmp) if len(tmp) < 169 else 169
            user_metadata[340: 340 + tmp_size] = tmp[0: tmp_size]

            # get file contents, truncate data if required
            try:
                with open(description_filename, 'r', encoding='utf8') as f:
                    text = f.read()
                    text_tmp = bytearray()
                    text_tmp.extend(map(ord, text))
                    data_tmp = np.array(text_tmp)
                    data = data_tmp.view(np.int8)
                    data_size = len(data) if len(data) < 511 else 511
                    user_metadata[1536: 1536 + data_size] = data[0: data_size]
            except OSError as e:
                print(e)
                # TODO: should be a stronger error
                print("Error opening description file.")

        # add user metadata (2048 bytes) to metadata_cmd
        metadata_cmd[user_metadata_index: user_metadata_index + 2048] = user_metadata

        # Metadata command reply: 'c' 'm' 'd' '0x80' + random + error
        metadata_cmd_reply = array.array('b', [0] * (4 + int32_size + int32_size))

        # send metadata_cmd and get it's reply
        try:
            res_write = self._dev.write(0x01, metadata_cmd.tobytes(), 100)
        except usb.core.USBError as e:
            # TODO: we probably should try again
            print("something went wrong while writing to the device")
            return

        assert res_write == len(metadata_cmd)

        try:
            ret = self._dev.read(0x81, metadata_cmd_reply, 1000)
        except usb.core.USBError as e:
            # TODO: we probably should try again
            traceback.print_exc()
            print("something went wrong while reading from the device")
            return

        # get the random received and the error received from the reply command
        rand_val_received = int.from_bytes(metadata_cmd_reply[4: 4 + int32_size], byteorder='little', signed=True)
        error_received = int.from_bytes(metadata_cmd_reply[8: 8 + int32_size], byteorder='little', signed=False)

        assert rand_val_received == rand_val[0]
        assert error_received == 0

        # prepare command to send and to receive
        # Data command length:     'c' 'm' 'd' '0x81' + random + dataIndex + 32768 + 'f'
        data_cmd = np.zeros(4 + int32_size + int32_size + 32768 + 1, dtype=np.int8)
        data_cmd_data_index = 4 + int32_size + int32_size

        data_cmd[0] = ord('c')
        data_cmd[1] = ord('m')
        data_cmd[2] = ord('d')
        data_cmd[3] = 0x81
        data_cmd[-1] = ord('f')

        # Data command reply:     'c' 'm' 'd' '0x81' + random + error
        data_cmd_reply = array.array('b', [0] * (4 + int32_size + int32_size))

        # loop to send the rest of the commands
        # check reply for each command sent
        for i in range(1, commands_to_send):
            # it has to be as an np.array of int32 so that we can get a view as int8s
            rand_val = np.random.randint(-32768, 32768, size=1, dtype=np.int32)
            # copy that random data
            data_cmd[4: 4 + int32_size] = rand_val.view(np.int8)

            # write dataIndex to the data_cmd (2 ints size)
            data_cmd[8: 8 + int32_size] = np.array([i], dtype=np.int32).view(np.int8)

            # write data from wave_int to cmd
            wave_idx = i * 32768
            data_block = wave_int8[wave_idx: wave_idx + 32768]
            data_cmd[data_cmd_data_index: data_cmd_data_index + len(data_block)] = data_block

            # send data to device
            try:
                res_write = self._dev.write(0x01, data_cmd.tobytes(), 100)
            except usb.core.USBError as e:
                # TODO: we probably should try again
                print("something went wrong while writing to the device")
                return

            # TODO: we probably should try again
            assert res_write == len(data_cmd)

            try:
                ret = self._dev.read(0x81, data_cmd_reply, 400)
            except usb.core.USBError as e:
                # TODO: we probably should try again
                traceback.print_exc()
                print("something went wrong while reading from the device")
                return

            # get the random received and the error received from the reply command
            rand_val_received = int.from_bytes(data_cmd_reply[4: 4 + int32_size], byteorder='little', signed=True)
            error_received = int.from_bytes(data_cmd_reply[8: 8 + int32_size], byteorder='little', signed=False)

            assert rand_val_received == rand_val[0]
            assert error_received == 0

    def __get_metadata_from_device(self, sound_index):
        int32_size = np.dtype(np.int32).itemsize

        # Read metadata command length: 'c' 'm' 'd' '0x84' + random + soundIndex + 'f'
        read_metadata_cmd = np.zeros(4 + int32_size + int32_size + 1, dtype=np.int8)
        read_metadata_cmd[0] = ord('c')
        read_metadata_cmd[1] = ord('m')
        read_metadata_cmd[2] = ord('d')
        read_metadata_cmd[3] = 0x84
        read_metadata_cmd[-1] = ord('f')

        rand_val = np.random.randint(-32768, 32768, size=1, dtype=np.int32)
        # copy that random data
        read_metadata_cmd[4: 4 + int32_size] = rand_val.view(np.int8)
        read_metadata_cmd[8: 8 + int32_size] = np.array([sound_index], dtype=np.int32).view(np.int8)

        # prepare to send command and receive the reply
        read_reply_cmd = array.array('b', [0] * (4 + 6 * int32_size + 2048))

        try:
            res_write = self._dev.write(0x01, read_metadata_cmd.tobytes(), 100)
        except usb.core.USBError as e:
            # TODO: we probably should try again
            print("something went wrong while writing to the device")
            return

        assert res_write == len(read_metadata_cmd)

        try:
            ret = self._dev.read(0x81, read_reply_cmd, 100)
        except usb.core.USBError as e:
            # TODO: we probably should try again
            traceback.print_exc()
            print("something went wrong while reading from the device")
            return

        metadata = collections.namedtuple('Metadata',
                                          ['metadata_array', 'description', 'bit_mask', 'sound_length', 'data_type',
                                           'sample_rate', 'sound_filename', 'metadata_filename',
                                           'description_filename',
                                           'has_sound', 'has_metadata', 'has_description'])

        # get data from the reply array
        metadata.metadata_array = array.array('b', [0] * 1024)
        metadata.description = array.array('b', [0] * 512)

        # get the random received and the error received from the reply command
        rand_val_received = int.from_bytes(read_reply_cmd[4: 4 + int32_size], byteorder='little', signed=True)
        error_received = int.from_bytes(read_reply_cmd[8: 8 + int32_size], byteorder='little', signed=False)

        assert rand_val_received == rand_val[0]
        assert error_received == 0

        # bitmask
        metadata.bit_mask = int.from_bytes(read_reply_cmd[12:12 + int32_size + int32_size], byteorder='little',
                                           signed=True)

        metadata.has_sound = metadata.bit_mask & (1 << sound_index) == (1 << sound_index)

        metadata.sound_length = int.from_bytes(read_reply_cmd[16:16 + int32_size], byteorder='little', signed=True)
        metadata.sample_rate = int.from_bytes(read_reply_cmd[20:20 + int32_size], byteorder='little', signed=True)
        metadata.data_type = int.from_bytes(read_reply_cmd[24:24 + int32_size], byteorder='little', signed=True)

        metadata.sound_filename = read_reply_cmd[28:170].tobytes().strip(b'\0')

        metadata.has_metadata = False
        metadata.metadata_filename = ''
        if read_reply_cmd[28 + 170]:
            metadata.has_metadata = True
            metadata.metadata_array[0:1024] = read_reply_cmd[28 + 512:28 + 512 + 1024]
            metadata.metadata_filename = read_reply_cmd[28 + 170: 28 + 170 + 170].tobytes().strip(b'\0')

        metadata.has_description = False
        metadata.description_filename = ''
        if read_reply_cmd[28 + 170 + 170]:
            metadata.has_description = True
            metadata.description[0:512] = read_reply_cmd[28 + 512 + 1024:28 + 512 + 1024 + 512]
            metadata.description_filename = read_reply_cmd[28 + 170 + 170: 28 + 170 + 170 + 170].tobytes().strip(b'\0')

        return metadata




BUF_SIZE = 6553600

def calculate_hash(self, filepath):

    sha1 = hashlib.sha1()
    with open(filepath, 'rb') as f:
        data = f.read(BUF_SIZE)
        sha1.update(data)

    return os.path.basename(filepath), 'sha1(firsts {0} bytes)'.format(BUF_SIZE), sha1.hexdigest()


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import os

# see pyforms_generic_editor.resources.__init__ for generic icons


def path(filename):
    return os.path.join(os.path.dirname(__file__), 'icons', filename)


BOARD_SMALL_ICON = path('board.png')

BOARDS_SMALL_ICON = path('boards.png')
BOX_SMALL_ICON = path('box.png')

SUBJECT_SMALL_ICON = path('subject.png')
SUBJECTS_SMALL_ICON = path('subjects.png')
UPLOAD_SMALL_ICON = path('upload.png')

EXPERIMENT_SMALL_ICON = path('experiment.png')
EXPERIMENTS_SMALL_ICON = path('experiments.png')

TASK_SMALL_ICON = path('task.png')
TASKS_SMALL_ICON = path('tasks.png')

CODEFILE_SMALL_ICON = path('codefile.png')
OTHERFILE_SMALL_ICON = path('otherfile.png')

PERSON_SMALL_ICON = path('person.png')
PERSONS_SMALL_ICON = path('persons.png')

REFRESH_SMALL_ICON = path('refresh.png')


# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

import pyforms as app
from pybpodgui_plugin.models.setup import Setup
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText
from pyforms.controls import ControlButton
from pybpodgui_api.models.experiment import Experiment
from pybpodgui_api.models.project import Project

logger = logging.getLogger(__name__)


class ExperimentWindow(Experiment, BaseWidget):
    """
    Define here which fields from the board model should appear on the details section.

    The model fields shall be defined as UI components like text fields, buttons, combo boxes, etc.

    You may also assign actions to these components.

    **Private attributes**

    _name
        Field to edit experiment name

        :type: :class:`pyforms.controls.ControlText`

    _task
        Combo box of available tasks. Current selected task is the task associated for this experiment
        and all its setups. Selecting a different task fires the event :class:`ExperimentWindow._ExperimentWindow__task_changed_evt`.

        :type: :class:`pyforms.controls.ControlCombo`

    **Methods**

    """

    def __init__(self, project):
        # type: (Project) -> None
        """

        :param project: project where this experiment belongs
        :type project: pycontrolgui.models.project.Project
        """
        BaseWidget.__init__(self, 'Experiment')
        self.layout().setContentsMargins(5, 10, 5, 5)

        self._name = ControlText('Exp. name')
        # self._task     = ControlCombo('Protocol', changed_event=self.__task_changed_evt)
        self._runsetup = ControlButton('Run all')

        self._formset = [
            '_name',
            # '_task',
            '_runsetup',
            ' '
        ]

        Experiment.__init__(self, project)

        # self.reload_tasks()

        self._name.changed_event = self.__name_changed_evt
        self._runsetup.value = self.__run_all

    def __run_all(self):
        for setup in self.setups:
            setup._run_task()

    def __task_changed_evt(self):
        """

        This methods is called every time the user presses the button.
        """
        # self.task = self._task.value
        # self.update_ui()

    def __name_changed_evt(self):
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._name.value

    def __sub__(self, obj):
        res = super().__sub__(obj)

        if isinstance(obj, Setup):
            for subject in self.project.subjects:
                subject.reload_setups()

        return res

    @property
    def name(self):
        return self._name.value

    @name.setter
    def name(self, value):
        self._update_name = True  # Flag to avoid recurse calls when editing the name text field
        self._name.value = value
        self._update_name = False


if __name__ == "__main__":
    # Execute the application
    app.start_app(ExperimentWindow)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from pybpodapi.state_machine.conditions import Conditions
from pybpodapi.state_machine.global_counters import GlobalCounters
from pybpodapi.state_machine.global_timers import GlobalTimers

from pybpodapi.bpod.hardware.events import EventName
from pybpodapi.bpod.hardware.output_channels import OutputChannel
from pybpodapi.bpod.hardware.channels import ChannelName

logger = logging.getLogger(__name__)


class StateMachineBase(object):
    """
    Each Bpod trial is programmed as a virtual finite state machine. This ensures precise timing of events - for any
    state machine you program, state transitions will be completed in less than 250 microseconds - so inefficient
    coding won't reduce the precision of events in your data.

    .. warning:: A lot of data structures are kept here for compatibility with original matlab library which are not
                 so python-like. Anyone is welcome to enhance this class but keep in mind that it will affect the whole
                 pybpodapi library.

    :ivar Hardware hardware: bpod box hardware description associated with this state machine
    :ivar Channels channels: bpod box channels handling
    :ivar list(str) state_names: list that holds state names added to this state machine
    :ivar list(float) state_timers: list that holds state timers
    :ivar int total_states_added: holds all states added, even if name is repeated
    :ivar list(int) state_timer_matrix: TODO:
    :ivar Conditions conditions: holds conditions
    :ivar GlobalCounters global_counters: holds global timers
    :ivar GlobalTimers global_timers: holds global counters
    :ivar list(tuple(int)) input_matrix: TODO:
    :ivar list(str) manifest: list of states names that have been added to the state machine
    :ivar list(str) undeclared: list of states names that have been referenced but not yet added
    :ivar tuple(str) meta_output_names: TODO:
    :ivar list(tuple(int)) output_matrix: TODO:
    :ivar bool is_running: whether this state machine is being run on bpod box

    """

    def __init__(self, bpod):
        """

        :param Hardware hardware: hardware description associated with this state machine
        """
        self.hardware = bpod.hardware  # type: Hardware

        self.state_names = []  # type: list(str)
        self.state_timers = [0] * self.hardware.max_states  # list(float)
        self.total_states_added = 0  # type: int

        # state change conditions
        self.state_timer_matrix = [0] * self.hardware.max_states
        self.conditions = Conditions(
            self.hardware.max_states, self.hardware.n_conditions
        )
        self.global_counters = GlobalCounters(
            self.hardware.max_states, self.hardware.n_global_counters
        )
        self.global_timers = GlobalTimers(
            self.hardware.max_states, self.hardware.n_global_timers
        )
        self.input_matrix = [[] for i in range(self.hardware.max_states)]

        # should be incremented whenever the user uses a timer
        self.n_global_timers_used = 0

        # should be incremented whenever the user uses a counter
        self.n_global_counters_used = 0

        # should be incremented whenever the user uses a conditions
        self.n_global_conditions_used = 0

        # if active uses the state 255 to store the previous state, so the user can go back in the state machine
        self.use_255_back_signal = False

        # List of states that have been added to the state machine
        self.manifest = []  # type: list(str)

        # List of states that have been referenced but not yet added
        self.undeclared = []  # type:list(str)

        # output actions
        self.output_matrix = [[] for i in range(self.hardware.max_states)]

        self.is_running = False

    def add_state(
        self, state_name, state_timer=0, state_change_conditions={}, output_actions=()
    ):
        """
        Adds a state to an existing state matrix.

        :param str name: A character string containing the unique name of the state. The state will automatically be assigned a number for internal use and state synchronization via the sync port
        :param float timer: The state timer value, given in seconds. This value must be zero or positive, and can range between 0-3600s. If set to 0s and linked to a state transition, the state will still take ~100us to execute the state's output actions before the transition completes
        :param dict state_change_conditions: Dictionary whose keys are names of a valid input event (state change) and values are names of states to enter if the previously listed event occurs (or 'exit' to exit the matrix and return all captured data)
        :param list(tuple) output_actions: a list of binary tuples where first value should contain the name of a valid output action and the second value should contain the value of the previously listed output action (see output actions for valid values).        

        Example:

        .. code-block:: python

            sma.add_state(
                state_name='Port1Lit',
                state_timer=.25,
                state_change_conditions={'Tup': 'Port3Lit', 'GlobalTimer1_End': 'exit'},
                output_actions=[('PWM1', 255)])

        """

        # TODO: WHY DO WE NEED THIS IF-ELSE?
        if state_name not in self.manifest:
            self.state_names.append(state_name)
            self.manifest.append(state_name)
            state_name_idx = len(self.manifest) - 1
        else:
            state_name_idx = self.manifest.index(state_name)
            self.state_names[state_name_idx] = state_name

        self.state_timer_matrix[state_name_idx] = state_name_idx

        self.state_timers[state_name_idx] = state_timer

        for event_name, event_state_transition in state_change_conditions.items():
            try:
                event_code = self.hardware.channels.event_names.index(event_name)
                logger.debug("Event code: %s", event_code)
            except:
                raise SMAError(
                    "Error creating state: "
                    + state_name
                    + ". "
                    + event_name
                    + " is an invalid event name."
                )

            if event_state_transition in self.manifest:
                destination_state_number = self.manifest.index(event_state_transition)
            else:
                if event_state_transition in ["exit", ">exit"]:
                    destination_state_number = float("NaN")

                elif event_state_transition in ["back", ">back"]:
                    self.use_255_back_signal = True
                    destination_state_number = 255
                else:  # Send to an undeclared state (replaced later with actual state in myBpod.sendStateMachine)
                    self.undeclared.append(event_state_transition)
                    destination_state_number = (len(self.undeclared) - 1) + 10000

            if EventName.is_state_timer(event_name):
                self.state_timer_matrix[state_name_idx] = destination_state_number

            elif EventName.is_condition(event_name):
                self.conditions.matrix[state_name_idx].append(
                    (event_code, destination_state_number)
                )

            elif EventName.is_global_counter_end(event_name):
                self.global_counters.matrix[state_name_idx].append(
                    (event_code, destination_state_number)
                )

            elif EventName.is_global_timer_trigger(event_name):

                if isinstance(event_state_transition, str):
                    v = int(event_state_transition, 2)
                else:
                    v = event_state_transition
                self.global_timers.end_matrix[state_name_idx] = v

            elif EventName.is_global_timer_cancel(event_name):

                if isinstance(event_state_transition, str):
                    v = int(event_state_transition, 2)
                else:
                    v = event_state_transition

                self.global_timers.end_matrix[state_name_idx] = v

            elif EventName.is_global_timer_end(event_name):
                self.global_timers.end_matrix[state_name_idx].append(
                    (event_code, destination_state_number)
                )

            elif EventName.is_global_timer_start(event_name):
                self.global_timers.start_matrix[state_name_idx].append(
                    (event_code, destination_state_number)
                )

            else:
                self.input_matrix[state_name_idx].append(
                    (event_code, destination_state_number)
                )

        for action_name, action_value in output_actions:
            if action_name == "Valve":
                output_code = self.hardware.channels.output_channel_names.index(
                    OutputChannel.Valve + str(action_value)
                )
                output_value = 1

                """
                elif action_name == 'ValveState':
                    output_code  = self.hardware.channels.output_channel_names.index( OutputChannel.Valve+str(action_value))
                    output_value = math.pow(2, action_value - 1)
                """
            elif action_name == OutputChannel.LED:
                output_code = self.hardware.channels.output_channel_names.index(
                    ChannelName.PWM + str(action_value)
                )
                output_value = 255

            else:
                try:
                    output_code = self.hardware.channels.output_channel_names.index(
                        action_name
                    )
                except:
                    raise SMAError(
                        "Error creating state: "
                        + state_name
                        + ". "
                        + action_name
                        + " is an invalid output name."
                    )

                output_value = action_value

            if action_name == OutputChannel.GlobalCounterReset:
                self.global_counters.reset_matrix[output_value] = 1

            # For backwards compatability, integers specifying global timers convert to equivalent binary decimals.
            # To specify binary, use a string of bits.
            if (
                output_code
                == self.hardware.channels.events_positions.globalTimerTrigger
            ):
                self.global_timers.triggers_matrix[state_name_idx] = 2 ** (
                    output_value - 1
                )

            if output_code == self.hardware.channels.events_positions.globalTimerCancel:
                self.global_timers.cancels_matrix[output_value - 1] = 1

            self.output_matrix[state_name_idx].append((output_code, output_value))

        self.total_states_added += 1

    def set_global_timer_legacy(self, timer_id=None, timer_duration=None):
        """
        Set global timer (legacy version)

        :param int timer_ID:
        :param float timer_duration: timer duration in seconds
        """
        self.global_timers.timers[timer_id - 1] = timer_duration

    def set_global_timer(
        self,
        timer_id,
        timer_duration,
        on_set_delay=0,
        channel=None,
        on_message=1,
        off_message=0,
        loop_mode=0,
        loop_intervals=0,
        send_events=1,
        oneset_triggers=None,
    ):
        """
        Sets the duration of a global timer. Unlike state timers, global timers can be triggered from any state (as an
        output action), and handled from any state (by causing a state change).

        :param int timer_ID: the number of the timer you are setting (an integer, 1-5).
        :param float timer_duration: the duration of the timer, following timer start (0-3600 seconds)
        :param float on_set_delay:
        :param str channel: channel/port name Ex: 'PWM2'
        :param int on_message:
        """
        timer_channel_idx = 255
        if channel is not None:
            try:
                timer_channel_idx = self.hardware.channels.output_channel_names.index(
                    channel
                )  # type: int
            except:
                raise SMAError(
                    "Error: {0} is an invalid output channel name.".format(channel)
                )

        index = timer_id - 1

        self.global_timers.timers[index] = timer_duration
        self.global_timers.on_set_delays[index] = on_set_delay
        self.global_timers.channels[index] = timer_channel_idx
        self.global_timers.on_messages[index] = on_message
        self.global_timers.off_messages[index] = off_message

        self.global_timers.loop_mode[index] = loop_mode
        self.global_timers.loop_intervals[index] = loop_intervals
        self.global_timers.send_events[index] = send_events

        if len(self.global_timers.onset_matrix) < index:
            for i in range(len(self.global_timers.onset_matrix), index + 1):
                self.global_timers.onset_matrix.append(0)

        if oneset_triggers is not None:
            self.global_timers.onset_matrix[index] = oneset_triggers

    def set_global_counter(
        self, counter_number=None, target_event=None, threshold=None
    ):
        """
        Sets the threshold and monitored event for one of the 5 global counters. Global counters can count instances of
        events, and handle when the count exceeds a threshold from any state (by triggering a state change).

        :param int counter_number: the number of the counter you are setting (an integer, 1-5).
        :param str target_event: port where to listen for event to count
        :param int threshold: number of times that should be count until trigger timer
        """
        event_code = self.hardware.channels.event_names.index(target_event)
        self.global_counters.attached_events[counter_number - 1] = event_code
        self.global_counters.thresholds[counter_number - 1] = threshold

    def set_condition(self, condition_number, condition_channel, channel_value):
        """
        Set condition

        :param int condition_number:
        :param str condition_channel:
        :param int channel_value:
        """
        channel_code = self.hardware.channels.input_channel_names.index(
            condition_channel
        )
        self.conditions.channels[condition_number - 1] = channel_code
        self.conditions.values[condition_number - 1] = channel_value


class SMAError(Exception):
    pass



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import os


def path(filename):
	return os.path.join(os.path.dirname(__file__), 'icons', filename)


ADD_SMALL_ICON = path('add.png')
NEW_SMALL_ICON = path('new.png')
OPEN_SMALL_ICON = path('open.png')
SAVE_SMALL_ICON = path('save.png')
EXIT_SMALL_ICON = path('exit.png')
CLOSE_SMALL_ICON = path('close.png')
USER_SETTINGS_ICON = path('settings.png')

PLAY_SMALL_ICON = path('play.png')
BUSY_SMALL_ICON = path('busy.png')
PROJECT_SMALL_ICON = path('project.png')
REMOVE_SMALL_ICON = path('remove.png')

APP_LOGO_ICON = path('cf-original.png')



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
from serial.tools import list_ports
from AnyQt import QtGui
from AnyQt.QtWidgets import QApplication

from confapp import conf

import pyforms as app
from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText, ControlCombo
from pyforms.controls import ControlButton
from pyforms.controls import ControlList
from pyforms.controls import ControlCheckBoxList
from pyforms.controls import ControlNumber

from pybpodgui_api.models.board import Board


from pybpodapi.bpod import Bpod

logger = logging.getLogger(__name__)


class BoardWindow(Board, BaseWidget):
    """
    Define here which fields from the board model should appear on the details section.

    The model fields shall be defined as UI components like text fields, buttons, combo boxes, etc.

    You may also assign actions to these components.

    **Properties**

        name
            :class:`string`

            Name associated with this board. Returns the current value stored in the :py:attr:`_name` text field.

        serial_port
            :class:`string`

            Serial port associated with this board. Returns the current value stored in the :py:attr:`_serial_port` text field.


    **Private attributes**

        _name
            :class:`pyforms.controls.ControlText`

            Text field to edit board name. Editing this field fires the event :meth:`BoardWindow._BoardWindow__name_changed_evt`.

        _serial_port
            :class:`pyforms.controls.ControlText`

            Text field to edit serial port. Editing this field fires the event :meth:`BoardWindow._BoardWindow__serial_changed_evt`.

        _log_btn
            :class:`pyforms.controls.ControlButton`

            Button to show this board events on a console window. Pressing the button fires the event :class:`BoardDockWindow.open_log_window`.

        _formset
            Describe window fields organization to PyForms.

    **Methods**

    """

    def __init__(self, project=None):
        """

        :param project: project where this board belongs
        :type project: pycontrolgui.models.project.Project
        """
        BaseWidget.__init__(self, 'Board')
        self.layout().setContentsMargins(5, 10, 5, 5)

        self._name = ControlText('Box name')
        self._serial_port = ControlCombo('Serial port')
        self._refresh_serials = ControlButton('',
                                              icon=QtGui.QIcon(conf.REFRESH_SMALL_ICON),
                                              default=self.__refresh_serials_pressed,
                                              helptext="Press here to refresh the list of available devices.")
        self._log_btn = ControlButton('Console')
        self._active_bnc = ControlCheckBoxList('BNC')
        self._active_wired = ControlCheckBoxList('Wired')
        self._active_behavior = ControlCheckBoxList('Behavior')
        self._loadports_btn = ControlButton('Load board info')
        self._netport = ControlNumber('Net port', default=36000+len(project.boards), minimum=36000, maximum=36100)
        self._events = ControlList('Events', readonly=True)
        self._inputchannels = ControlList('Input channels', readonly=True)
        self._outputchannels = ControlList('Output channels', readonly=True)

        self._saved_serial_port = None

        Board.__init__(self, project)

        self._formset = [
            '_name',
            ('_serial_port', '_refresh_serials'),
            '_netport',
            '_log_btn',
            '=',
            '_loadports_btn',
            {
                'Ports': [
                    'Enabled or disable ports',
                    '_active_bnc',
                    '_active_wired',
                    '_active_behavior',
                ],
                'Events':[
                    '_events'
                ],
                'Input ch.':[
                    '_inputchannels'
                ],
                'Output ch.':[
                    '_outputchannels'
                ]
            }
        ]
        self._name.changed_event = self.__name_changed_evt
        self._loadports_btn.value = self.__load_bpod_ports

        self._fill_serial_ports()

    def _fill_serial_ports(self):
        self._serial_port.add_item('', '')
        for n, port in enumerate(sorted(list_ports.comports()), 1):
            self._serial_port.add_item("{device}".format(device=port.device), str(port.device))

    def freegui(self):
        QApplication.processEvents()

    def stop_thread(self):
        self._timer.stop()

    def __load_bpod_ports(self):
        # present error if no serial port is selected
        if not self._serial_port.value:
            self.warning("Please select a serial port before proceeding.", "No serial port selected")
            return

        if "not connected" in self._serial_port.text:
            self.warning("Please connect the device to the computer before proceeding.", "Device not connected")
            return

        try:
            bpod = Bpod(self._serial_port.value)
            hw = bpod.hardware
            # load the ports to the GUI ###############################
            self._active_bnc.value = [('BNC{0}'.format(j+1),  True) for j, i in enumerate(hw.bnc_inputports_indexes)]
            self._active_wired.value = [('Wire{0}'.format(j+1), True) for j, i in enumerate(hw.wired_inputports_indexes)]
            if len(self._active_behavior.value) == 0:
                self._active_behavior.value = [('Port{0}'.format(j+1), True) for j, i in enumerate(hw.behavior_inputports_indexes)]
            #############################################################
            self._events.value = [["{0} ({1})".format(x, i)] for i, x in enumerate(hw.channels.event_names)]
            self._inputchannels.value = [[x] for x in hw.channels.input_channel_names]
            self._outputchannels.value = [[x] for x in hw.channels.output_channel_names]

            bpod.close()
        except Exception as e:
            self.critical(str(e), 'Error loading ports')

    def __refresh_serials_pressed(self):
        tmp = self._serial_port.value

        self._serial_port.clear()
        self._fill_serial_ports()

        self.serial_port = self._saved_serial_port
        self.serial_port = tmp

    def __name_changed_evt(self):
        """
        React to changes on text field :py:attr:`_name`.

        This methods is called every time the user changes the field.
        """
        if not hasattr(self, '_update_name') or not self._update_name:
            self.name = self._name.value

    @property
    def name(self):
        return self._name.value

    @name.setter
    def name(self, value):
        self._update_name = True  # Flag to avoid recursive calls when editing the name text field
        self._name.value = value
        self._update_name = False

    @property
    def serial_port(self):
        return self._serial_port.value

    @serial_port.setter
    def serial_port(self, value):
        # if the option isn't available in the self._serial_port we probably should add it (and remove it later when
        # the device is connected)
        if value is not None and (value, value) not in self._serial_port.items:
            self._serial_port.add_item("{val} (not connected)".format(val=value), value)
            if not hasattr(self, "_save_serial_port"):
                self._saved_serial_port = value
        self._serial_port.value = value

    @property
    def net_port(self):
        return int(self._netport.value)

    @net_port.setter
    def net_port(self, value):
        if value is not None:
            self._netport.value = value

    @property
    def enabled_bncports(self):
        return [b for v, b in self._active_bnc.items] if self._active_bnc.count > 0 else None

    @enabled_bncports.setter
    def enabled_bncports(self, value):
        Board.enabled_bncports.fset(self, value)
        if value is None:
            self._active_bnc.value = []
        else:
            self._active_bnc.value = [('BNC{0}'.format(j+1), v) for j, v in enumerate(value)]

    @property
    def enabled_wiredports(self):
        return [b for v, b in self._active_wired.items] if self._active_wired.count > 0 else None

    @enabled_wiredports.setter
    def enabled_wiredports(self, value):
        Board.enabled_wiredports.fset(self, value)

        if value is None:
            self._active_wired.value = []
        else:
            self._active_wired.value = [('Wire{0}'.format(j+1), v) for j, v in enumerate(value)]

    @property
    def enabled_behaviorports(self):
        return [b for v, b in self._active_behavior.items] if self._active_behavior.count > 0 else None

    @enabled_behaviorports.setter
    def enabled_behaviorports(self, value):
        Board.enabled_behaviorports.fset(self, value)
        if value is None:
            self._active_behavior.value = []
        else:
            self._active_behavior.value = [('Port{0}'.format(j+1), v) for j, v in enumerate(value)]


# Execute the application
if __name__ == "__main__":
    app.start_app(BoardWindow)



# !/usr/bin/python3
# -*- coding: utf-8 -*-

import logging

from confapp import conf
from pybpodgui_api.exceptions.run_setup import RunSetupError
from pybpodgui_api.models.setup.setup_io import SetupBaseIO

logger = logging.getLogger(__name__)


class SetupCom(SetupBaseIO):
    """
    Define board actions that are triggered by setup.

    **Properties**

        status
            :class:`int`

            Holds setup status depending on board communication state.

    **Methods**

    """

    #### SETUP STATUS CONSTANTS ####
    STATUS_READY = 0
    STATUS_BOARD_LOCKED = 1  # The setup is free but the board is busy
    STATUS_RUNNING_TASK = 2  # The setup is busy running the task, but it cannot be stopped yet

    def __init__(self, experiment):
        super(SetupCom, self).__init__(experiment)
        self.status = self.STATUS_READY

    ##########################################################################
    ####### PROPERTIES #######################################################
    ##########################################################################

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    ##########################################################################
    ####### FUNCTIONS ########################################################
    ##########################################################################
    def stop_trial(self):
        if self.status == self.STATUS_RUNNING_TASK:
            self.board.stop_trial()

    def stop_task(self):
        if self.status == self.STATUS_RUNNING_TASK:
            self.board.stop_task()

    def kill_task(self):
        if self.status == self.STATUS_RUNNING_TASK:
            self.board.kill_task()

    def pause_trial(self):
        if self.status == self.STATUS_RUNNING_TASK:
            self.board.pause_trial()

    def resume_trial(self):
        if self.status == self.STATUS_RUNNING_TASK:
            self.board.resume_trial()

    def run_task(self):
        """
        Run task on board.

        In order to run task, the project must be saved before.
        This method will restore task variables from session, create a new session
        and start the 'run task' operation by calling board function run_task.

        """
        if not self.can_run_task():
            return

        try:
            # update the status of the setup
            self.status = self.STATUS_RUNNING_TASK

            session = self.create_session()
            session.user = self.project.loggeduser

            self._run_flag = self.board.run_task(
                session,
                self.board_task,
                self.path,
                detached=self.detached
            )

            for s in self.subjects:
                s += session

            # we need this if we want to put the play button on the subject treenode's session correctly
            self.project.update_ui()

        except Exception as err:
            logger.error(str(err), exc_info=True)
            raise Exception("Unknown error found while running task. See log for more details.")

    def can_run_task(self):
        if not self.board or not self.task:
            logger.warning("Setup has no protocol assigned.")
            raise RunSetupError("Please assign a board and protocol first")
        if conf.PYBPODGUI_API_CHECK_SUBJECTS_ON_RUN and len(self.subjects) == 0:
            logger.warning("No Subjects selected")
            raise RunSetupError("Please add subjects to this experiment")
        if self.project.loggeduser is None:
            logger.warning("No User selected")
            raise RunSetupError("Please select an User. Please double click an User in the project tree to select it.")
        if not self.project.is_saved():
            # check conf property and save project automatically and return true
            if conf.PYBPODGUI_API_AUTO_SAVE_PROJECT_ON_RUN:
                logger.info("Auto saving project")
                print("Auto saving project")
                self.project.save()
                return True
            else:
                logger.warning("Run protocol cannot be executed because project is not saved.")
                raise RunSetupError("Project must be saved before run protocol")
        return True



import logging
import os

import pybpodgui_api
from pybpod_alyx_module.alyx_details import AlyxDetails
from pybpodgui_plugin.models.subject import SubjectUIBusy
from sca.formats import json

logger = logging.getLogger(__name__)


class AlyxSubject(SubjectUIBusy):

    def __init__(self, project):
        super(AlyxSubject, self).__init__(project)

    def add_alyx_info(self, jsondata):
        self.name = jsondata.get('nickname', None)
        self.alyx_id = jsondata.get('id', None)
        self._get_alyx_members_from_json(jsondata)

        if not hasattr(self, '_alyx_menu'):
            self._alyx_menu = self._tree.add_popup_menu_option('Alyx Details', self.showdetails, item=self.node)
        self._name.readonly = True

    def save(self):
        """
        Save subject data on filesystem.

        :ivar str project_path: Project path.  
        :return: Dictionary containing the setup info to save.  
        :rtype: dict
        """
        if not hasattr(self, 'alyx_id'):
            super().save()
            return

        if not self.name:
            logger.warning("Skipping subject without name")
            return None
        else:  
            if not os.path.exists(self.path): os.makedirs(self.path)

            if self.data:
                data = self.data
            else:
                data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url ='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a subject used on PyBpod GUI.',
                )

            self._save_alyx_member_to_json(data)

            config_path = os.path.join(self.path, self.name+'.json')
            with open(config_path, 'w') as fstream: json.dump(data, fstream)

    def toJSON(self):
        if not hasattr(self, 'alyx_id'):
            return super().toJSON()

        data = json.scadict(
                    uuid4_id=self.uuid4,
                    software='PyBpod GUI API v'+str(pybpodgui_api.__version__),
                    def_url ='http://pybpod.readthedocs.org',
                    def_text='This file contains information about a subject used on PyBpod GUI.',
                )

        data['name'] = self.name
        data['uuid4'] = self.uuid4
        self._save_alyx_member_to_json(data)

        return json.dumps(data)

    def load(self, path):
        """
        Load subject data from filesystem

        :ivar str subject_path: Path of the subject
        :ivar dict data: data object that contains all subject info
        """
        self.name  = os.path.basename(path)

        try:
            with open( os.path.join(self.path, self.name+'.json'), 'r' ) as stream:
                self.data = data = json.load(stream)

            if data.get('alyx_id', None) is None:
                super().load(path)
                return

            self.uuid4 = data.uuid4 if data.uuid4 else self.uuid4
            self.alyx_id = data.get('alyx_id', None)
            self._get_alyx_members_from_json(data)

            self._alyx_menu = self._tree.add_popup_menu_option('Alyx Details', self.showdetails, item=self.node)
            self._name.readonly = True

        except:
            raise Exception('There was an error loading the configuration file for the subject [{name}]. File not found.'.format(name=self.name))

    def _get_alyx_members_from_json(self, data):
        self.alyx_nickname = data.get('nickname', None)
        self.alyx_url = data.get('url', None)
        self.alyx_responsible_user = data.get('responsible_user', None)
        self.alyx_birth_date = data.get('birth_date', None)
        self.alyx_age_weeks = data.get('age_weeks', None)
        self.alyx_death_date = data.get('death_date', None)
        self.alyx_species = data.get('species', None)
        self.alyx_sex = data.get('sex', None)
        self.alyx_litter = data.get('litter', None)
        self.alyx_strain = data.get('strain', None)
        self.alyx_source = data.get('source', None)
        self.alyx_line = data.get('line', None)
        self.alyx_projects = data.get('projects', [])
        self.alyx_lab = data.get('lab', None)
        self.alyx_genotype = data.get('genotype', [])
        self.alyx_description = data.get('description', None)
        self.alyx_alive = data.get('alive', None)
        self.alyx_weighings = data.get('weighings', [])
        self.alyx_water_administrations = data.get('water_administrations', [])
        self.alyx_reference_weight = data.get('reference_weight', None)
        self.alyx_last_water_restriction = data.get('last_water_restriction', None)
        self.alyx_expected_water = data.get('expected_water', None)
        self.alyx_remaining_water = data.get('remaining_water', None)

    def _save_alyx_member_to_json(self, data):
        data['nickname'] = self.alyx_nickname
        data['url'] = self.alyx_url
        data['alyx_id'] = self.alyx_id
        data['responsible_user'] = self.alyx_responsible_user
        data['birth_date'] = self.alyx_birth_date
        data['age_weeks'] = self.alyx_age_weeks
        data['death_date'] = self.alyx_death_date
        data['species'] = self.alyx_species
        data['sex'] = self.alyx_sex
        data['litter'] = self.alyx_litter
        data['strain'] = self.alyx_strain
        data['source'] = self.alyx_source
        data['line'] = self.alyx_line
        data['projects'] = self.alyx_projects
        data['lab'] = self.alyx_lab
        data['genotype'] = self.alyx_genotype
        data['description'] = self.alyx_description
        data['alive'] = self.alyx_alive
        data['weighings'] = self.alyx_weighings
        data['water_administration'] = self.alyx_water_administrations
        data['reference_weight'] = self.alyx_reference_weight
        data['last_water_restriction'] = self.alyx_last_water_restriction
        data['expected_water'] = self.alyx_expected_water
        data['remaining_water'] = self.alyx_remaining_water

    def create_treenode(self, tree):
        """
        Creates node for this board under the parent "Boards" node.

        This methods is called when the board is first created.

        The following actions get assigned to node:
            * *Remove*: :meth:`BoardTreeNode.remove`.

        Sets key events:
            * :meth:`BoardTreeNode.node_key_pressed_event`


        :param tree: the project tree
        :type tree: pyforms.controls.ControlTree
        :return: new created node
        :return type: QTreeWidgetItem
        """
        super().create_treenode(tree)

        # save the tree so we can add the pop-up on load and add_alyx_info for those subjects that require it
        self._tree = tree

        return self.node

    def showdetails(self):
        if not hasattr(self, 'detailswindow'):
            self.detailswindow = AlyxDetails(self)
        self.detailswindow.show()
        return self.detailswindow

    def collect_data(self, data):
        _data = super().collect_data(data)

        if not hasattr(self, 'alyx_id'):
            return _data

        _data.update({'nickname': self.alyx_nickname})
        _data.update({'url': self.alyx_url})
        _data.update({'alyx_id': self.alyx_id})
        _data.update({'responsible_user': self.alyx_responsible_user})
        _data.update({'birth_date': self.alyx_birth_date})
        _data.update({'age_weeks': self.alyx_age_weeks})
        _data.update({'death_date': self.alyx_death_date})
        _data.update({'species': self.alyx_species})
        _data.update({'sex': self.alyx_sex})
        _data.update({'litter': self.alyx_litter})
        _data.update({'strain': self.alyx_strain})
        _data.update({'source': self.alyx_source})
        _data.update({'line': self.alyx_line})

        _data.update({'projects': []})
        for item in self.alyx_projects:
            _data['projects'].append(item)

        _data.update({'lab': self.alyx_lab})

        _data.update({'genotype': []})
        for item in self.alyx_genotype:
            _data['genotype'].append(item)

        _data.update({'description': self.alyx_description})
        _data.update({'alive': self.alyx_alive})

        _data.update({'weighings': []})
        for item in self.alyx_weighings:
            _data['weighings'].append(str(item))

        _data.update({'water_administrations': []})
        for item in self.alyx_water_administrations:
            _data['water_administrations'].append(str(item))

        _data.update({'reference_weight': self.alyx_reference_weight})
        _data.update({'last_water_restriction': self.alyx_last_water_restriction})
        _data.update({'expected_water': self.alyx_expected_water})
        _data.update({'remaining_water': self.alyx_remaining_water})

        return _data
